#include "odbc.h"

void odbc_connexion(conn_info* conn, const char* source, const char* user, const char* passwd){
  long        V_OD_erg; // result of functions
  char        V_OD_stat[10];		// Status SQL
  SQLINTEGER  V_OD_err;
  SQLSMALLINT V_OD_mlen;
  char        V_OD_msg[200];  // 1. allocate Environment handle and register version 
  V_OD_erg=SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&conn->env);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error AllocHandle\n");
    exit(0);
  }
  V_OD_erg=SQLSetEnvAttr(conn->env, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0); 
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error SetEnv\n");
    SQLFreeHandle(SQL_HANDLE_ENV, conn->env);
    exit(0);
  }
  // 2. allocate connection handle, set timeout
  V_OD_erg = SQLAllocHandle(SQL_HANDLE_DBC, conn->env, &conn->hdbc); 
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error AllocHDB %d\n",V_OD_erg);
    SQLFreeHandle(SQL_HANDLE_ENV, conn->env);
    exit(0);
  }
  SQLSetConnectAttr(conn->hdbc, SQL_LOGIN_TIMEOUT, (SQLPOINTER *)5, 0);
  // 3. Connect to the datasource "web" 
  /*V_OD_erg = SQLConnect(conn->hdbc, (SQLCHAR*) "web", SQL_NTS,
    (SQLCHAR*) "christa", SQL_NTS,
    (SQLCHAR*) "", SQL_NTS);*/
  V_OD_erg = SQLConnect(conn->hdbc, (SQLCHAR*) source, SQL_NTS,
			(SQLCHAR*) user, SQL_NTS,
			(SQLCHAR*) passwd, SQL_NTS);	
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error SQLConnect %d\n",V_OD_erg);
    SQLGetDiagRec(SQL_HANDLE_DBC, conn->hdbc,1, 
		  V_OD_stat, &V_OD_err,V_OD_msg,100,&V_OD_mlen);
    printf("%s (%d)\n",V_OD_msg,V_OD_err);
    SQLFreeHandle(SQL_HANDLE_ENV, conn->env);
    exit(0);
  }
  //printf("Connected !\n");
}

void odbc_deconnexion(conn_info* conn) {
  SQLDisconnect(conn->hdbc);
  SQLFreeHandle(SQL_HANDLE_DBC,conn->hdbc);
  SQLFreeHandle(SQL_HANDLE_ENV, conn->env);
}

int odbc_prepare_requete(comp_curs* cc,const char* requ) {

  conn_info* conn=cc->conn;
  SQLHSTMT* stmt=cc->stmt;
  stmt_cols* cols=cc->cols;
  int nbcols=cc->nbcols;
  
  long V_OD_erg; // result of functions
  char			 V_OD_stat[10];		// Status SQL
  SQLINTEGER		 V_OD_rowanz,V_OD_id;
  SQLSMALLINT		 V_OD_mlen;
  char             V_OD_msg[200],V_OD_buffer[200];
  int i;
  SQLINTEGER   V_OD_err; 

  V_OD_erg=SQLAllocHandle(SQL_HANDLE_STMT, conn->hdbc, stmt);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error in AllocStatement %d\n",V_OD_erg);
    SQLGetDiagRec(SQL_HANDLE_DBC, conn->hdbc,1, V_OD_stat,&V_OD_err,V_OD_msg,100,&V_OD_mlen);
    printf("%s (%d)\n",V_OD_msg,V_OD_err);
    return 0;
  }

  for (i=0;i<nbcols;i++) {
    SQLBindCol(*stmt,i+1,cols[i].type,cols[i].data,cols[i].taille,cc->lenbuff);
  }
  
  V_OD_erg=SQLExecDirect(*stmt,(SQLCHAR*)requ,SQL_NTS);   
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error in Select %d\n",V_OD_erg);
    SQLGetDiagRec(SQL_HANDLE_DBC, conn->hdbc,1, V_OD_stat,&V_OD_err,V_OD_msg,100,&V_OD_mlen);
    printf("%s (%d)\n",V_OD_msg,V_OD_err);
    SQLFreeHandle(SQL_HANDLE_STMT,*stmt);
    return 0;
  }

  V_OD_erg=SQLRowCount(*stmt,&V_OD_rowanz);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Number ofRowCount %d\n",V_OD_erg);
    SQLFreeHandle(SQL_HANDLE_STMT,*stmt);
    return 0;
  }
  printf("Number of Rows %d\n",V_OD_rowanz);

  return 1;
}

/* attention : chaque attribut est tronque a 20 caracteres maxi */
char * tuple_suivant(char* tuple,comp_curs * cc) {
  SQLHSTMT* stmt=cc->stmt;
  SQLRETURN ret;
  int i;
  char* temp=(char*)malloc(512); // prevu large au cas ou...
  strcpy(tuple,"");
  //  if(SQLFetch(cc->stmt) != SQL_NO_DATA) {
  if((ret=SQLFetch(*stmt)) == SQL_SUCCESS) {
    for (i=0;i<cc->nbcols;i++) {
      strcat(tuple,"|");
      if (cc->cols[i].type==SQL_C_CHAR) {
	sprintf(temp,"%20s",(char*)(cc->cols[i].data));
      } else {
	sprintf(temp,"%10d",(int)(*((int*)(cc->cols[i].data))));
      }
      strcat(tuple,temp);
    }
    strcat(tuple,"|");
  } else {
    switch (ret) {
    case SQL_ERROR :
      printf("erreur\n");
      break;
    case SQL_INVALID_HANDLE :
      printf("Erreur handle\n");
      break;
    }
  }
  free(temp);
  return tuple;
}

char* odbc_entetes(char* entetes,comp_curs* cc) {
  int i;
  char * temp=(char*)malloc(512);
  strcpy(entetes,"");
  for (i=0;i<cc->nbcols;i++) {
    strcat(entetes,"|");
    if (cc->cols[i].type==SQL_C_CHAR) {
      sprintf(temp,"%20s",(char*)(cc->cols[i].nom));
    } else {
      sprintf(temp,"%10s",(char*)(cc->cols[i].nom));
    }
    strcat(entetes,temp);
  }
  strcat(entetes,"|");
  free(temp);
  return entetes;
}

void odbc_termine_requete(SQLHSTMT stmt) {
  SQLFreeHandle(SQL_HANDLE_STMT,stmt);
}

int odbc_requete(conn_info* conn, const char* requ) {
  long V_OD_erg; // result of functions
char			 V_OD_stat[10];		// Status SQL
SQLINTEGER		 V_OD_err,V_OD_rowanz,V_OD_id;
SQLSMALLINT		 V_OD_mlen,V_OD_colanz;
char             V_OD_msg[200],V_OD_buffer[200];
SQLHSTMT V_OD_hstmt;   // Handle for a statement

  V_OD_erg=SQLAllocHandle(SQL_HANDLE_STMT, conn->hdbc, &V_OD_hstmt);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error in AllocStatement %d\n",V_OD_erg);
    SQLGetDiagRec(SQL_HANDLE_DBC, conn->hdbc,1, V_OD_stat,&V_OD_err,V_OD_msg,100,&V_OD_mlen);
    printf("%s (%d)\n",V_OD_msg,V_OD_err);
    return -1;
  }
  SQLBindCol(V_OD_hstmt,1,SQL_C_CHAR, V_OD_buffer,150,&V_OD_err);
  SQLBindCol(V_OD_hstmt,2,SQL_C_ULONG,&V_OD_id,150,&V_OD_err);
  
  V_OD_erg=SQLExecDirect(V_OD_hstmt,(SQLCHAR*)requ,SQL_NTS);   
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Error in Select %d\n",V_OD_erg);
    SQLGetDiagRec(SQL_HANDLE_DBC, conn->hdbc,1, V_OD_stat,&V_OD_err,V_OD_msg,100,&V_OD_mlen);
    printf("%s (%d)\n",V_OD_msg,V_OD_err);
    SQLFreeHandle(SQL_HANDLE_STMT,V_OD_hstmt);
    return -1;
  }

  V_OD_erg=SQLNumResultCols(V_OD_hstmt,&V_OD_colanz);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    SQLFreeHandle(SQL_HANDLE_STMT,V_OD_hstmt);
    return -1;
  }

  printf("Number of Columns %d\n",V_OD_colanz);
  V_OD_erg=SQLRowCount(V_OD_hstmt,&V_OD_rowanz);
  if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
    printf("Number ofRowCount %d\n",V_OD_erg);
    SQLFreeHandle(SQL_HANDLE_STMT,V_OD_hstmt);
    return -1;
  }

  printf("Nombre de lignes %d\n",V_OD_rowanz);
  V_OD_erg=SQLFetch(V_OD_hstmt);  
  while(V_OD_erg != SQL_NO_DATA) {
    //printf("Result: %d %s\n",V_OD_id,V_OD_buffer);
    V_OD_erg=SQLFetch(V_OD_hstmt);  
  }

  SQLFreeHandle(SQL_HANDLE_STMT,V_OD_hstmt);

}


comp_curs * create_comp_curs(comp_curs * cc, char * node, char * table, char * user, char * passwd) {
  long V_OD_erg; // result of functions
  SQLINTEGER V_OD_rowanz;
  char* query;
  int i;
  //stmt_cols* cols;
  //int nbcols;
  cc->conn=(conn_info*)malloc(sizeof(conn_info));
  cc->stmt=(SQLHSTMT*)malloc(sizeof(SQLHSTMT));
  cc->lenbuff=(SQLINTEGER*)malloc(sizeof(SQLINTEGER));

  // Connexion
  odbc_connexion(cc->conn,node,user,passwd);

  // Recup de la structure de la table
  cc->nbcols=get_structure(cc->conn->hdbc,&(cc->cols),table);

  // Recup des tuples
  query=(char*)malloc(200);
  strcpy(query,"SELECT ");
  if (cc->nbcols>0) {
    strcat(query,cc->cols[0].nom);
  }
  for (i=1;i<cc->nbcols;i++) {
    strcat(query,",");
    strcat(query,cc->cols[i].nom);
  }
  strcat(query," FROM ");
  strcat(query,table);
  //printf("requete : %s\n",query);
  //odbc_prepare_requete(cc->conn,cc->stmt,query,cc->cols,cc->nbcols);
  odbc_prepare_requete(cc,query);
  free(query);

  //printf("******%d****\n",SQLFetch(cc->stmt));
  //printf("avant retour\n");
  return cc;
}

void close_comp_curs(comp_curs * cc) {
  SQLFreeHandle(SQL_HANDLE_STMT,cc->stmt);
  odbc_deconnexion(cc->conn);
}

// le nombre maxi de colonnes est 30
int get_structure(SQLHDBC hdbc,stmt_cols **columns,char* table) {

  SQLRETURN cliRC = SQL_SUCCESS;
  int cpt = 0;
  SQLHANDLE hstmt; /* statement handle */

  stmt_cols* cols=(stmt_cols*)malloc(30*sizeof(stmt_cols));

  struct
  {
    SQLINTEGER ind;
    SQLCHAR val[129];
  }
  colName, colTypeText;

  struct
  {
    SQLINTEGER ind;
    SQLINTEGER val;
  }
  colLength, colType;

  /* criteria to look for */
  SQLCHAR tbSchemaPattern[] = "%";
  SQLCHAR *tbNamePattern = table;
  SQLCHAR colNamePattern[] = "%";

  /* allocate a statement handle */
  cliRC = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  //DBC_HANDLE_CHECK(hdbc, cliRC);

  /* get column information for a table */
  cliRC = SQLColumns(hstmt,
                     NULL,
                     0,
                     tbSchemaPattern,
                     SQL_NTS,
                     tbNamePattern,
                     SQL_NTS,
                     colNamePattern,
                     SQL_NTS);
  //STMT_HANDLE_CHECK(hstmt, hdbc, cliRC);

  /* bind column 4 to variable */
  cliRC = SQLBindCol(hstmt, 4, SQL_C_CHAR, colName.val, 129, &colName.ind);
  //STMT_HANDLE_CHECK(hstmt, hdbc, cliRC);
  //cliRC = SQLBindCol(hstmt, 6, SQL_C_CHAR, colTypeText.val, 129, &colTypeText.ind);

/* bind column 5 to variable */
  cliRC = SQLBindCol(hstmt, 5, SQL_C_LONG,(SQLPOINTER)&colType.val, sizeof(colType.val), &colType.ind);
  //STMT_HANDLE_CHECK(hstmt, hdbc, cliRC);

  /* bind column 7 to variable */
  cliRC = SQLBindCol(hstmt,
                     7,
                     SQL_C_LONG,
                     (SQLPOINTER)&colLength.val,
                     sizeof(colLength.val),
                     &colLength.ind);
  //STMT_HANDLE_CHECK(hstmt, hdbc, cliRC);

  /* fetch next row */
  cliRC = SQLFetch(hstmt);
  //STMT_HANDLE_CHECK(hstmt, hdbc, cliRC);

  while (cliRC != SQL_NO_DATA_FOUND) {
    /*printf("colonne suivante");
    if (colType.val==SQL_C_CHAR) {
      printf("(type : chaine) %s \n",colTypeText.val);   // cast a voir
    } else {
      printf("(type : nombre) %s \n",colTypeText.val);   // cast a voir
    }
    */
    cols[cpt].nom=malloc(strlen(colName.val)+1);
    strcpy(cols[cpt].nom,colName.val);
    cols[cpt].taille=colLength.val;
    switch (colType.val) {
    case SQL_VARCHAR :
      cols[cpt].type=SQL_C_CHAR;
      cols[cpt].data=(char*)malloc(cols[cpt].taille);
      break;
    case SQL_INTEGER :
      cols[cpt].type=SQL_C_SLONG;
      cols[cpt].data=(int*)malloc(sizeof(int));
      break;
    default : // supposons que c'est du texte...
      cols[cpt].type=SQL_C_CHAR;
      cols[cpt].data=(char*)malloc(cols[cpt].taille);
    }
    cpt++;
    cliRC = SQLFetch(hstmt);
  }

  cols=(stmt_cols*)realloc(cols,sizeof(stmt_cols)*cpt);
  *columns=cols;
  //trace_structure(cols,cpt);
  return cpt;
}


void trace_structure(stmt_cols* cols,int nbc){
  int i;
  for (i=0;i<nbc;i++) {
    printf("colonne %s de type %d de taille %d\n",cols[i].nom,cols[i].type,cols[i].taille);
  }
}

/*int main(int argc,char *argv[]) {
  comp_curs cc;
  char* entetes=(char*) malloc(200);
  char* tuple=(char*)malloc(1000);
  create_comp_curs(&cc,"LIFO","Fonction");
  printf("entetes : %s\n",odbc_entetes(entetes,&cc));
  tuple=tuple_suivant(tuple,&cc);
  while (strlen(tuple)>0) {
    printf("%s\n",tuple);
    tuple=tuple_suivant(tuple,&cc);
  }
  close_comp_curs(&cc);
  free(tuple);
  free(entetes);
  return(0);
}
*/

/*
int main(int argc,char *argv[]) {
  conn_info maConnexion;
  stmt_cols *cols;
  SQLHSTMT stmt;
  int nbcols;
  int valeur;
  

  odbc_connexion(&maConnexion,"***","***","***");

comp_curs * create_comp_curs(comp_curs * cc, char * node, char * table) {

  nbcols=get_structure(maConnexion.hdbc,&cols,"Personne");

  trace_structure(cols,nbcols);

  // recup du contenu...

  odbc_deconnexion(&maConnexion);

  return(0);
}
*/

