
/* Dynamic hashing of record with different key-length */


#ifndef _hist_h
#define _hist_h
#include <stdio.h>
#include <sys/types.h>

#define MALLOC_SIZE        8192
#define HIST_BUFFER_SIZE   8192

/* indice des frequences des tables tables table1 et table2  par rapport a une frequence seuil */
#define LEFT_INDEX           1     /*  (freq2> seuil) et (freq2> freq1)       */
#define RIGHT_INDEX          2     /*  (freq1> seuil) et (freq1> freq2)       */
#define HASH_INDEX           3     /*  (freq1 <= seuil) et (freq2<= seuil)    */

#define SEP_CHAR           '\n'
#define SEP_LENGTH           1
#define PID                  5
        
typedef unsigned char      uchar;


typedef struct hist_dynamic_array {
  char *buffer;
  uint elements,max_element;
  uint alloc_increment;
  uint size_of_element;
} HIST_DYNAMIC_ARRAY;


typedef struct st_hist {
  uint key_offset,key_length;		  /* Length of key if const length */
  uint records,blength,current_record;
  HIST_DYNAMIC_ARRAY array;		  /* Place for hist_keys */
  char *(*get_key)(const char *,uint*);
  void (*free)(void *);
} HIST;

typedef struct st_hist_info {
  uint next;			     /* index to next key           */
  char *data;	                     /* data for current entry      */
  uint freq ;    	             /* frequency for current entry */
} HIST_LINK;


/***** MATTHIEU : modif HIST_ARRAY : n'est plus pointeur *********/
typedef struct st_hist_array {
  char *data;	                    
  struct st_hist_array  *next ;      
} HIST_ARRAY;

  
int      hist_init(HIST *hist,uint size,uint key_offset,uint key_length,
             char *(*get_key)(const char *,uint*), void (*free_element)(void*));

char*    hist_element(HIST *hist,ulong index);
char*    hist_search(HIST *info,const char *key,uint length);
char*    hist_next(HIST *info,const char *key,uint length);
void     hist_free(HIST *tree);
void     hist_show(HIST *hist);
void     free_record(void *record);
int      hist_insert(HIST *info,const char *data);
int      hist_insert_frequency(HIST *info,const char *data,uint frequency);
int      hist_add_entry(HIST *info,const char *data);
int      hist_add_entry_frequency(HIST *info,const char *data,uint frequency);
int      hist_delete(HIST *hist,char *record);
int      hist_update(HIST *hist,char *record,char *old_key,uint old_key_length);

// transformation du format HIST_ARRAY au format HIST*
/*****  Enleve par MAtthieu int hist_load_array_buffer(HIST_ARRAY *list,HIST *hist,uint size,uint key_offset,uint key_length,
			   char* (*get_key)(const char *,uint*), void (*free_element)(void*));
*******/

char*    get_key(const char*,uint *);
char*    get_trunc_key(const char *,uint *);

int hist_array_buffer_show(HIST_ARRAY *list);
int hist_array_buffer_show_file(HIST_ARRAY *list,char* file);


/***********MATTHIEU ***********/
int ha_nb_blocs(HIST_ARRAY *list);
HIST_ARRAY* ha_nouveau(HIST_ARRAY* ha);
void ha_libere(HIST_ARRAY** list);
int ha_estvide(HIST_ARRAY* ha); 
HIST_ARRAY* ha_ajout_couple(HIST_ARRAY *ha,const char* str,int val);
HIST_ARRAY* ha_ajout_triplet(HIST_ARRAY *ha,const char* str,int val1,int val2);
HIST_ARRAY* ha_extract_couple(HIST_ARRAY* ha,char* str,int* val,int* offset);
HIST_ARRAY* ha_extract_triplet(HIST_ARRAY *ha,char* str,int* val1,int* val2,int* offset);
HIST_ARRAY* HIST_to_HIST_ARRAY(HIST *hist);
HIST_ARRAY* HIST_to_HIST_ARRAY_proc(HIST *hist,int proc);
void HIST_ARRAY_to_HIST(HIST_ARRAY* list,HIST *hist,uint size,uint key_offset,uint key_length,
                                      char* (*get_key)(const char *,uint*), void (*free_element)(void*));
void HIST_ARRAY_to_HIST_proc(HIST_ARRAY* list,HIST* hist,uint size,uint key_offset,uint key_length,
                                      char* (*get_key)(const char *,uint*), void (*free_element)(void*));


//#define dynamic_element(array,index,type) ((type)((array)->buffer) +(index))
//#define dynamic_array_ptr(array,index) ((array)->buffer+(index)*(array)->size_of_element)
#define array_dynamic_element(array,index,type) ((type)((array)->buffer) +(index))
#define array_dynamic_array_ptr(array,index) ((array)->buffer+(index)*(array)->size_of_element)

#define init_ptr(x)     x=0
#define max(a, b)	((a) > (b) ? (a) : (b))
#define min(a, b)	((a) < (b) ? (a) : (b))
#endif


