#include <stdio.h>
#include <stdlib.h>
#include "bloc.h"



/*******************************/
/*                             */
/* Manipulations sur les blocs */
/*                             */
/*******************************/


/*************************************************************/
/* ajouter une table dans un fichier (ordre lexicographique) */
/*************************************************************/

void AddTabFile (C_table* T) {
 
  FILE *INFILE, *OUTFILE;

  int TROUVE = 0; // pour indiquer qd on a trouve ou inserer la nouvelle table

  bloc B1,B2;
  L_attr* A;
  L_mach* M;

  /* on cree le bloc header de la table */
  B2.type = theader;
  motcopy(B2.data.head.nomtab, T->head.nomtab);
  motcopy(B2.data.head.nombase, T->head.nombase);
  B2.data.head.htup = T->head.htup;
  B2.data.head.hoct = T->head.hoct;

  /* Ouverture du fichier existant en lecture */
  INFILE = fopen(NOMFICH, "r"); 
  
  /* Ouverture du fichier temporaire en ecriture */
  OUTFILE = fopen(FICHTMP, "w"); 
  
  /* Traitement */
  fread(&B1, sizeof (bloc), 1,INFILE);                        // on essaie de sortir le premier bloc s'il existe
  if (!feof(INFILE)) {                                        // si il existe un bloc
    if (motcmp(B1.data.head.nomtab, B2.data.head.nomtab) > 0) // qui porte un nom situe apres dans l'ordre lexicographique
      TROUVE = 1;                                             // on a trouve l'endroit ou inserer la nouvelle table
    else {                                                    // il s'agit d'un bloc header, mais pas celui que l'on cherche
      fwrite(&B1, sizeof (bloc), 1,OUTFILE);                  // on recopie le bloc dans le fichier temporaire
      fread(&B1, sizeof (bloc), 1,INFILE);                    // on passe au bloc suivant
    }   
  }
  else {                                                      // s'il n'y a pas encore de bloc dans le fichier
  }

  /* recopie des tables dont le nom precede lexicographiquement celui a inserer */
  while (!feof(INFILE) && !TROUVE) { // on n'est pas a la fin du fichier et on n'a pas encore trouve une table situee apres dans l'ordre lexicographique
    if (B1.type == theader) {                                   // si il s'agit d'un bloc de type header
      if (motcmp(B1.data.head.nomtab, B2.data.head.nomtab) > 0) // qui porte un nom situe apres dans l'ordre lexicographique
	TROUVE = 1;                                             // on a trouve l'endroit ou inserer la nouvelle table
      else {                                                    // il s'agit d'un bloc header, mais pas celui que l'on cherche
	  fwrite(&B1, sizeof (bloc), 1,OUTFILE);                // on recopie le bloc dans le fichier temporaire
	  fread(&B1, sizeof (bloc), 1,INFILE);                  // on passe au bloc suivant
      }
    }
    else {                                                      // si il ne s'agit pas d'un bloc header
      fwrite(&B1, sizeof (bloc), 1,OUTFILE);                    // on recopie le bloc dans le fichier temporaire
      fread(&B1, sizeof (bloc), 1,INFILE);                      // on passe au bloc suivant
    }
  }
  
  /* Ecriture du nouvel enregistrement : */

  /* 1) le header */
  fwrite(&B2, sizeof (bloc), 1,OUTFILE);
  
  /* 2) les attributs */
  A = T->P_attr; // on se place sur le premier attribut

  while (A != NULL) {                        // tant qu'il reste des attributs dans la liste
    B2.type = tattribut;                     // on cree le bloc attribut correspondant
    motcopy(B2.data.attr.nomattr, A->attr.nomattr);
    B2.data.attr.atyp = A->attr.atyp;
    
    fwrite(&B2, sizeof (bloc), 1,OUTFILE);   // on copie le bloc dans le fichier
    
    A = A->suivant;                          // on passe a l'attribut suivant
  }

  /* 3) les machines */
  M = T->P_mach; // on se place sur le premier attribut

  while (M != NULL) {                        // tant qu'il reste des machines dans la liste
    B2.type = tmachine;                      // on cree le bloc machine correspondant
    motcopy(B2.data.mach.nommach, M->mach.nommach);
    motcopy(B2.data.mach.nomloc,  M->mach.nomloc);
    B2.data.mach.mtup = M->mach.mtup;
    B2.data.mach.moct = M->mach.moct;

    fwrite(&B2, sizeof (bloc), 1,OUTFILE);   // on copie le bloc dans le fichier

    M = M->suivant;                          // on passe a la machine suivante
  }

  /* ecriture du dernier bloc lu */
  if (!feof(INFILE) && TROUVE) {
    fwrite(&B1, sizeof(bloc), 1,OUTFILE);
  }

  fread(&B1, sizeof (bloc), 1,INFILE);

  /* Copie du reste des enregistrements */
  while (!feof(INFILE)) {                  // tant qu'il reste des blocs  recopier
    fwrite(&B1, sizeof (bloc), 1,OUTFILE); // on les recopie
    fread(&B1, sizeof (bloc), 1,INFILE);
  }

  /* fermeture des fichiers */
  fclose(OUTFILE);
  fclose(INFILE);

  remove(NOMFICH);
  rename(FICHTMP,NOMFICH);
}


/********************/
/* vider le fichier */
/********************/

void ViderFile () {
 
  FILE *f_out;

  f_out = fopen(NOMFICH, "w");
  fclose(f_out);
}


/**********************************/
/* supprimer une table du fichier */
/**********************************/

void DelTabFile (Mot name, Mot nbase) {
 
  FILE *INFILE, *OUTFILE;

  int trouve = 0; // pour indiquer qd on a trouve la table que l'on cherche ou si on a depasse l'endroit ou elle devrait se trouver (car ordre lexicographique)
  int depasse = 0;

  bloc B1,B2;
  L_attr* A;
  L_mach* M;

  /* Ouverture du fichier existant en lecture */
  INFILE = fopen(NOMFICH, "r"); 

  /* Ouverture du fichier temporaire en ecriture */
  OUTFILE = fopen(FICHTMP, "w"); 
 
  /* Traitement */

  fread(&B1, sizeof (bloc), 1,INFILE);                           // on essaie de sortir le premier bloc s'il existe
  if (!feof(INFILE)) {                                       
    if ((motcmp(B1.data.head.nomtab, name) == 0) && (motcmp(B1.data.head.nombase, nbase) == 0)) {                                                                
      trouve = 1;                                                // on a trouve la table cherchee
    }
    else {                                                       // s'il ne s'agit pas de la table que l'on cherche
      if (motcmp(B1.data.head.nomtab, name) > 0) {
	depasse = 1;                                             // on est trop loin ds l'ordre lexicographique => table a effacer inexistante
      } 

      fwrite(&B1, sizeof (bloc), 1,OUTFILE);                     // on recopie le bloc dans le fichier temporaire
      fread(&B1, sizeof (bloc), 1,INFILE);                       // on passe au bloc suivant
    }    
  }
  else {                                                         // s'il n'y a pas encore de bloc dans le fichier
  }

  /* recopie des tables dont le nom precede lexicographiquement celle que l'on cherche */
  while (!feof(INFILE) && (!trouve) && (!depasse)) { // on n'est pas a la fin du fichier et on n'a pas encore trouve la table que l'on cherche
    if (B1.type == theader) {        // si il s'agit d'un bloc de type header
      if ((motcmp(B1.data.head.nomtab, name) == 0) && (motcmp(B1.data.head.nombase, nbase) == 0)) {
	trouve = 1;
      }
     
      else {                                      // s'il ne s'agit pas de la table que l'on cherche
	if (motcmp(B1.data.head.nomtab, name) > 0) {
	  depasse = 1;                            // on est trop loin ds l'ordre lexicographique => table a effacer inexistante
	} 
	fwrite(&B1,sizeof(bloc),1,OUTFILE);       // on recopie le bloc dans le fichier temporaire 
	fread(&B1,sizeof(bloc),1,INFILE);         // on passe au bloc suivant
      }
    }

    else {                                        // si il ne s'agit pas d'un bloc header
      fwrite(&B1,sizeof(bloc),1,OUTFILE);         // on recopie le bloc dans le fichier temporaire
      fread(&B1,sizeof(bloc),1,INFILE);           // on passe au bloc suivant
    }
  }
  
  /* on passe les blocs de la table que l'on veut effacer */

  trouve = 0;

  while (!feof(INFILE) && (!trouve) && (!depasse)) { // tant que l'on ne tombe pas sur un bloc header  - ie les blocs correspondent a la table que l'on supprime -
                                                     // et tant qu'il reste des blocs...
    fread(&B1,sizeof(bloc),1,INFILE);                // on passe au bloc suivant sans le recopier
    if (B1.type == theader) {
      trouve = 1;
    }
  }

  /* Copie du reste des enregistrements */
  while (!feof(INFILE)) {               // tant qu'il reste des blocs   
    fwrite(&B1,sizeof(bloc),1,OUTFILE); // on les recopie 
    fread(&B1,sizeof(bloc),1,INFILE);
  }

  /* Fermeture des fichiers */
  fclose(OUTFILE);
  fclose(INFILE);

  remove(NOMFICH);
  rename(FICHTMP,NOMFICH);
}


/*******************************************************************************/
/* extraire une C_table d'un fichier (avec suppression de la table du fichier) */
/*******************************************************************************/

void ExtractTabFile (C_table* T, Mot name, Mot nbase) {

 
  FILE *INFILE, *OUTFILE;
  
  int trouve = 0; // pour indiquer qd on a trouve la table que l'on cherche ou si on a depasse l'endroit ou elle devrait se trouver (car ordre lexicographique)
  int depasse = 0;
  
  bloc B1,B2;
  L_attr* A;
  L_mach* M;
  
  /* Ouverture du fichier existant en lecture */
  INFILE = fopen(NOMFICH, "r"); 
  
  /* Ouverture du fichier temporaire en ecriture */
  OUTFILE = fopen(FICHTMP, "w"); 
 
  /* Traitement */

  fread(&B1, sizeof (bloc), 1,INFILE);                           // on essaie de sortir le premier bloc s'il existe
  if (!feof(INFILE)) {                                       
    if ((motcmp(B1.data.head.nomtab, name) == 0) && (motcmp(B1.data.head.nombase, nbase) == 0)) {                                                                
      trouve = 1;                                                // on a trouve la table cherchee
    }
    else {                                                       // s'il ne s'agit pas de la table que l'on cherche
      if (motcmp(B1.data.head.nomtab, name) > 0) {
	depasse = 1;                                             // on est trop loin ds l'ordre lexicographique => table a effacer inexistante
      } 

      fwrite(&B1, sizeof (bloc), 1,OUTFILE);                     // on recopie le bloc dans le fichier temporaire
      fread(&B1, sizeof (bloc), 1,INFILE);                       // on passe au bloc suivant
    }    
  }
  else {                                                         // s'il n'y a pas encore de bloc dans le fichier
  }

  /* recopie des tables dont le nom precede lexicographiquement celle que l'on cherche */
  while (!feof(INFILE) && (!trouve) && (!depasse)) { // on n'est pas a la fin du fichier et on n'a pas encore trouve la table que l'on cherche
    if (B1.type == theader) {        // si il s'agit d'un bloc de type header
      if ((motcmp(B1.data.head.nomtab, name) == 0) && (motcmp(B1.data.head.nombase, nbase) == 0)) {
	trouve = 1;
      }
     
      else {                                      // s'il ne s'agit pas de la table que l'on cherche
	if (motcmp(B1.data.head.nomtab, name) > 0) {
	  depasse = 1;                            // on est trop loin ds l'ordre lexicographique => table a effacer inexistante
	} 
	fwrite(&B1,sizeof(bloc),1,OUTFILE);       // on recopie le bloc dans le fichier temporaire 
	fread(&B1,sizeof(bloc),1,INFILE);         // on passe au bloc suivant
      }
    }

    else {                                        // si il ne s'agit pas d'un bloc header
      fwrite(&B1,sizeof(bloc),1,OUTFILE);         // on recopie le bloc dans le fichier temporaire
      fread(&B1,sizeof(bloc),1,INFILE);           // on passe au bloc suivant
    }
  }
   
 
  if ((trouve) && (!depasse) && (strcasecmp(B1.data.head.nomtab, name) == 0)) { // si on a trouve la table a extraire
    /* Chargement de la table dans une C_table : */
    
    /* 1) le header */
    motcopy(T->head.nomtab, B1.data.head.nomtab);
    motcopy(T->head.nombase, B1.data.head.nombase);
    T->head.htup = B1.data.head.htup;
    T->head.hoct = B1.data.head.hoct;
    
    fread(&B1,sizeof(bloc),1,INFILE);

    /* 2) les attributs */
    trouve = 0;
    while (!feof(INFILE) && !trouve) {
      if(B1.type != tattribut) {                                  // si on trouve une machine ou un header
	trouve == 1;                                              // on a fini de lire les attributs
      }
      else {                                                      // s'il s'agit d'un attribut
	AddAttribut (T, B1.data.attr.nomattr, B1.data.attr.atyp); // on l'ajoute a la C_table
	fread(&B1,sizeof(bloc),1,INFILE);                         // on passe au bloc suivant
      }
    }
    
    /* 3) les machines */
    if (B1.type == tmachine) {                                                                         // si le dernier bloc extrait etait une machine
      trouve = 0;
      AddMachine (T, B1.data.mach.nommach, B1.data.mach.nomloc, B1.data.mach.mtup, B1.data.mach.moct); // on ajoute la machine a la C_table
      fread(&B1,sizeof(bloc),1,INFILE);                                                                // on sort le bloc suivant
    }
    
    while (!feof(INFILE) && !trouve) {     
      if(B1.type != tmachine) {    // si on trouve un header on a fini de lire les machines (on ne peut trouver d'attribut juste apres une machine)
	trouve == 1;
      }
      else {                                                                                             // s'il s'agit d'une machine
	AddMachine (T, B1.data.mach.nommach, B1.data.mach.nomloc, B1.data.mach.mtup, B1.data.mach.moct); // on l'ajoute a la C_table
	fread(&B1,sizeof(bloc),1,INFILE);                                                                // on sort le bloc suivant
      }
    }
  }

  /* Copie du reste des blocs */
  while (!feof(INFILE)) {
    fwrite(&B1,sizeof(bloc),1,OUTFILE);
    fread(&B1,sizeof(bloc),1,INFILE); 
  }

  /* Fermeture des fichiers */
  fclose(OUTFILE);
  fclose(INFILE);

  remove(NOMFICH);
  rename(FICHTMP,NOMFICH);
}

/***************************************************/
/* test de l'appartenance d'une table a un fichier */
/***************************************************/

int TabInFile (C_table* T, Mot name, Mot nbase) {
  
  FILE* INFILE;
  int res = 0; // pour stocker le resultat
  int TROUVE = 0;
  bloc B1;

  /* Ouverture du fichier existant en lecture */
  INFILE = fopen(NOMFICH, "r"); 

  fread(&B1,sizeof(bloc),1,INFILE);
  if (B1.type == theader) {                                   // si il s'agit d'un bloc de type header
    if ((motcmp(B1.data.head.nomtab, name) > 0) && (motcmp(B1.data.head.nombase, nbase)==0))
                                                              // qui porte un nom situe apres dans l'ordre lexicographique
      TROUVE = 1;                                             // plus la peine de chercher
    else
      if ((motcmp(B1.data.head.nomtab, name) == 0) && (motcmp(B1.data.head.nombase, nbase)==0)) { 
                                                              // on a trouve la table cherchee
	TROUVE = 1;
	res = 1;
      }
  }

  /* recherche de la table name */
  while (!feof(INFILE) && !TROUVE) { // on n'est pas a la fin du fichier et on n'a pas encore trouve la table que l'on cherche
    fread(&B1,sizeof(bloc),1,INFILE);
    if (B1.type == theader) {                                   // si il s'agit d'un bloc de type header
      if ((motcmp(B1.data.head.nomtab, name) > 0)  && (motcmp(B1.data.head.nombase, nbase)==0))
                                                                // qui porte un nom situe apres dans l'ordre lexicographique
	TROUVE = 1;                                             // plus la peine de chercher
      else
	if ((motcmp(B1.data.head.nomtab, name) == 0)  && (motcmp(B1.data.head.nombase, nbase)==0)){
                                                                // on a trouve la table cherchee
	  TROUVE = 1;
	  res = 1;
	}
    } 
  }

  /* Fermeture du fichier */
  fclose(INFILE);

  return res;
} 


/***********************************************/
/* comptage du nombre de blocs dans le fichier */
/***********************************************/

int NbBloc () {
  
  FILE* INFILE;
  int res = 0; // pour stocker le resultat
  bloc B1;

  /* Ouverture du fichier existant en lecture */
  INFILE = fopen(NOMFICH, "r"); 

  fread(&B1,sizeof(bloc),1,INFILE);

  /* recherche de la table name */
  while (!feof(INFILE)) { // on n'est pas a la fin du fichier et on n'a pas encore trouve la table que l'on cherche
    fread(&B1,sizeof(bloc),1,INFILE);
    res++;
  }
  
  /* Fermeture du fichier */
  fclose(INFILE);

  return res;
} 



/**********************************/
/*                                */
/* Manipulations sur les C_tables */
/*                                */
/**********************************/


/************************/
/* creer une table vide */
/************************/

C_table CreerTableVide (Mot name, Mot nbase, int ntup, int noct) {
  C_table T;

  motcopy(T.head.nomtab, name);
  motcopy(T.head.nombase, nbase);
  T.head.htup = ntup;
  T.head.hoct = noct;

  T.P_attr = NULL;
  T.D_attr = NULL;

  T.P_mach = NULL;
  T.D_mach = NULL;

  return T;
}


/*********************/
/* Effacer une Table */
/*********************/

void DelTable(C_table T) {
}


/**********************************/
/* ajouter un attribut a la table */
/**********************************/

void AddAttribut (C_table* T, Mot name, attrtype at) {

  attribut A;
  L_attr* Nouveau;
  L_attr* tmp;

  motcopy(A.nomattr, name);
  A.atyp = at;

  Nouveau = (L_attr*)malloc(sizeof(L_attr));

  Nouveau->attr = A;
  Nouveau->suivant = NULL;

  tmp = T->D_attr;
  if (tmp != NULL) {
    tmp->suivant = Nouveau; // erreur sur cette ligne
  }
  T->D_attr = Nouveau;

  if ((T->P_attr) == NULL) { // s'il s'agit du premier attribut insere
    T->P_attr = Nouveau;
  }

}


/**********************************/
/* enlever un attribut a la table */
/**********************************/

void DelAttribut (C_table* T, Mot name) {
  L_attr* prec;
  L_attr* cour;

  prec = NULL;
  cour = NULL;

  prec = T->P_attr;
  if (prec != NULL) {
    cour = prec->suivant;
    if ((cour == NULL)&&((prec->attr.nomattr) == name)) { // si il n'y a qu'un seul attribut : celui a effacer
      T->P_attr = NULL; // pas de premier attribut
      T->D_attr = NULL; // ni de dernier...
      free(prec);       // on libere l'espace occupe par l'attribut
    }
  }

  while ((cour != NULL)&&((cour->attr.nomattr) != name)) { // la liste
    prec = prec->suivant; // on parcours jusqu'a ce que cour pointe sur l'attribut a supprimer ou si on arrive en fin de liste
    cour = cour->suivant;
  }

  if (cour != NULL) {                // si on a trouve l'attribut a effacer
    prec->suivant = cour->suivant;
    if (cour->suivant == NULL) {     // si on efface le dernier attribut (cour)
      T->D_attr = prec;              // le dernier attribut de la liste est celui pointe par prec
    }
    free(cour);                      // on libere l'espace occupe par l'attribut
  }
}


/**********************************/
/* ajouter une machine a la table */
/**********************************/

void AddMachine (C_table* T, Mot nomM, Mot nomL, int nt, int no) {

  machine M;
  L_mach* Nouveau;
  L_mach* tmp;

  motcopy(M.nommach, nomM);
  motcopy(M.nomloc, nomL);
  M.mtup = nt;
  M.moct = no;

  Nouveau = (L_mach*)malloc(sizeof(L_mach));

  Nouveau->mach = M;
  Nouveau->suivant = NULL;

  tmp = T->D_mach;
  if (tmp != NULL) {
    tmp->suivant = Nouveau;
  }
  T->D_mach = Nouveau;

  if ((T->P_mach) == NULL) { // s'il s'agit de la premiere machine inseree
    T->P_mach = Nouveau;
  }

}


/**********************************/
/* enlever une machine a la table */
/**********************************/

void DelMachine (C_table* T, Mot name) {
  L_mach* prec;
  L_mach* cour;

  prec = NULL;
  cour = NULL;

  prec = T->P_mach;
  if (prec != NULL) {
    cour = prec->suivant;
    if ((cour == NULL)&&((prec->mach.nommach) == name)) { // si il n'y a qu'une seule machine : celle a effacer
      T->P_mach = NULL; // pas de premiere machine
      T->D_mach = NULL; // ni de derniere...
      free(prec);       // on libere l'espace occupe par la machine
    }
  }

  while ((cour != NULL)&&((cour->mach.nommach) != name)) { // la liste
    prec = prec->suivant; // on parcours jusqu'a ce que cour pointe sur la machine a supprimer ou si on arrive en fin de liste
    cour = cour->suivant;
  }

  if (cour != NULL) {                // si on a trouve la machine a effacer
    prec->suivant = cour->suivant;
    if (cour->suivant == NULL) {     // si on efface la derniere machine (cour)
      T->D_mach = prec;              // la derniere machine de la liste est celle pointee par prec
    }
    free(cour);                      // on libere l'espace occupe par la machine
  }
}


/*********************/
/*                   */
/* Fonctions Annexes */
/*                   */
/*********************/



/********************/
/* Longeur d'un Mot */
/********************/

int motlen (Mot m) {
  int res = 0;

  while (m[res]!='\0') {
    res++;
  }  
  return res;
}


/***************************/
/* Conversion char* en Mot */
/***************************/

int char2mot (char* s, Mot m) {
  int i;
  int longueur = strlen(s);

  for (i=1; i<=longueur; i++) {
    m[i-1]=s[i-1];
  }
  
  m[longueur]='\0';

  return 1;
}


/***************************/
/* Conversion Mot en char* */
/***************************/

int mot2char (Mot m, char* s) {
  
  int i;
  int longueur = motlen (m);

  for (i=1; i<=longueur; i++) {
    s[i-1]=m[i-1];
  }
  
  s[longueur]='\0';

  return 1;
}


/*************************/
/* Comparaison de 2 Mots */
/*************************/

int motcmp (Mot m1, Mot m2) {
  int res = 0;  // renvoie  0 si les 2 mots sont egaux
                //          1 si m1 > m2 
                //         -1 si m1 < m2 
  int pos = 0;  // rang de la lettre que l'on compare
  int fini = 0; // 0 si on n'a pas encore pu determiner l'ordre de classement

  while (!fini) {
    if (m1[pos] !='\0') {
      if (m2[pos] !='\0') {
	if (m1[pos]<m2[pos]) {
	  res = -1;
	}
	else
	  if (m1[pos]>m2[pos]) {
	    res = 1;
	  }
      }
      else { // m1[pos] !='\0' et m2[pos] =='\0'
	res = 1;
      }
    }
    else { // m1[pos] =='\0'
      if (m2[pos] !='\0') {
	res = -1;
      }
      else {
	fini = 1;
      }
    }
    if (res != 0) {
      fini = 1;
    }
    pos++;
  }

  return res;
}


/******************/
/* Copie d'un Mot */
/******************/

int motcopy (Mot m1, Mot m2) {
  int i;

  for (i=0; i<=MAXCHAR; i++) {
    m1[i]=m2[i];    
  }

  return 1;
}
