#ifndef _BLOC_H_
#define _BLOC_H_

#define NOMFICH "info.db" // nom du fichier dans lequel se trouve la bd
#define FICHTMP "temp.db" // nom du fichier temporaire dans lequel se trouve la bd (lors de l'ajout de tables...)
#define MAXCHAR 30


typedef char Mot[MAXCHAR+1];


/******************************************************************************/
/*                                                                            */
/* Structure d'une table enregistree dans un fichier                          */
/*                                                                            */
/* Il s'agit d'une suite de blocs.                                            */
/* Une table est composee :                                                   */
/*                             - d'un bloc header                             */
/*                             - d'un nombre quelconque de blocs attribut     */
/*                             - d'un nombre quelconque de blocs machine      */
/*                                                                            */
/* Quand on tombe sur un nouveau bloc header, il s'agit d'une nouvelle table. */
/*                                                                            */
/******************************************************************************/


/* Enumeration des differents types de blocs trouvables dans un fichier */
typedef enum {
  tindefini,
  theader,
  tattribut,
  tmachine
} bloctype;


/* Enumration des differents types des attributs */
typedef enum {
  indefini,
  text,
  num,
  date
} attrtype;


/* Structure des infos relatives a un header */
typedef struct {
  Mot nomtab;   // nom de la table
  Mot nombase;  // nom de la base dans laquelle se trouve la table (permet de differencier 2 tables portant le meme nom, mais dans des bases differentes)
  int htup;     // nb de tuples dans la table
  int hoct;     // taille en octets de la table
} header;


/* structure des infos relatives a un attribut */
typedef struct {
  Mot nomattr; // nom de l'attribut
  attrtype atyp; // type de l'attribut
} attribut;


/* structure des infos relatives a une machine */
typedef struct {
  Mot nommach;   // nom de la machine
  Mot nomloc;    // nom local de la machine
  int mtup;      // nb de tuples sur la machine
  int moct;      // taille des donnees sur la machine
} machine;


/* union des differents contenus possibles d'un bloc */
typedef union {
  header head;
  attribut attr;
  machine mach;
} blocdata;


/* structure d'un bloc contenu dans le fichier */
typedef struct {
  bloctype type; // type de bloc (header, attribut ou machine ?)
  blocdata data; // donnees contenues dans le bloc
} bloc;



/*****************************************************************/
/*                                                               */
/* Structure d'une table hors du fichier pour pouvoir la traiter */
/*                                                               */
/* Une table est une structure composee :                        */
/*                - un champ header                              */
/*                - un champ pointeur vers une liste d'attributs */
/*                - un champ pointeur vers une liste de machines */
/*                                                               */
/*****************************************************************/


/* liste chainee d'attributs */
typedef struct s_L_attr {
  attribut attr;
  struct s_L_attr* suivant;
} L_attr;


/* liste chainee de machines */
typedef struct s_L_mach {
  machine mach;
  struct s_L_mach* suivant;
} L_mach;


/* Structure d'une table */
typedef struct {
  header head;    // infos gnrales de la table
  L_attr* P_attr; // pointeur vers le premier attribut
  L_attr* D_attr; // pointeur vers le dernier attribut
  L_mach* P_mach; // pointeur vers la premire machine
  L_mach* D_mach; // pointeur vers la dernire machine
} C_table;



/*******************************/
/*                             */
/* Manipulations sur les blocs */
/*                             */
/*******************************/

void AddFirstTabFile (C_table* T);                     // ajouter la premiere table dans le fichier
void AddTabFile (C_table* T);                          // ajouter une table dans le fichier
void DelTabFile (Mot name, Mot nbase);                 // supprimer une table du fichier

void ExtractTabFile (C_table* T, Mot name, Mot nbase); // extraire une table d'un fichier en la "chargeant" dans une C_table

int TabInFile (C_table* T, Mot name, Mot nbase);        // teste si une table nommee name est dans le fichier

int NbBloc ();                                         // compte le nb de blocs dans le fichier (tous types confondus)

void ViderFile ();                                     // compte le nb de blocs dans le fichier (tous types confondus)


/**********************************/
/*                                */
/* Manipulations sur les C_tables */
/*                                */
/**********************************/

C_table CreerTableVide (Mot name, Mot nbase, int ntup, int noct); // creation d'une nouvelle table vide
void DelTable(C_table T);                                         // liberation de l'espace alloue a une table

void AddAttribut (C_table* T, Mot name, attrtype at);             // ajouter un attribut a une table
void DelAttribut (C_table* T, Mot name);                          // supprimer un attribut d'une table

void AddMachine (C_table* T, Mot nomM, Mot nomL, int nt, int no); // ajouter une machine a une table
void DelMachine (C_table* T, Mot name);                           // supprimer une machine d'une table

#endif
