#include "common.h"
#include "decompose.h"
#include <string.h>
#include <stdlib.h>

/*************************************************************************************************
 ** Ce fichier contient les primitives transformant les chaines de caracteres produites par le  **
 ** programme yacc (calc.y) en listes d'operations et d'attributs utilisables par la suite du   **
 ** programme (c'est a dire pour construire l'arbre)                                            **
 ************************************************************************************************/


/* retourne le nom de table et le nom d'attribut, sachant qu'au depart on a nomtabel.nomattribut, ou NULL et attribut si le format est simplifie 
*/
void attrEtTable(char* chaine,char** attr,char** table) {
  int rangPoint=0;
  int rangFin;
  int taille=strlen(chaine);
  while ((rangPoint<taille) && (chaine[rangPoint] != '.')
	 && (chaine[rangPoint] != ',') && (chaine[rangPoint] != ']')) {
    rangPoint++;
  }
  if ((rangPoint<taille) &&(chaine[rangPoint]=='.')) { // format table.attr
    *table=recopien(chaine,rangPoint);
  
    rangFin=++rangPoint;
    while ((rangFin<taille)&&(chaine[rangFin] != ',') && (chaine[rangFin] != ']')) {
      rangFin++;
    }
    *attr=recopien(chaine+rangPoint,rangFin-rangPoint);
  } else { // format attr seul ou valeur (selection)
    *table=NULL;
    *attr=recopien(chaine,rangPoint);
  }
}


/* retourne le nombre d'operation elementaires dans la clause where.
   On utilise le fait que chaque operation commence par le caractere '['.
   parametre : where = la chaine de caracteres decrivant la clause where
   generee par calc.y
   retour : le nombre d'operation elementaires cntenues dans cette chaine
*/
int getNbNoeuds(char* where){
  char *parcours=where;
  int nbn=0;
  while ((parcours=index(parcours,'['))!=NULL) {
    parcours++;
    nbn++;
  }
  return nbn;
}
int getOperations(char* wherestr,list_op** whereop) {
  /* on commence par crer un ensemble de noeuds independants correspondant chacun a une operation elementaire */
  list_op* ops;
  int cpt;
  int rang;
  int nbNoeuds;
  char* whereloc;
  char* tok;
  char* attr;
  char* table;
  char* tempo;

  //printf("%s\n",wherestr);
  nbNoeuds=getNbNoeuds(wherestr);
  /* TODO : cas de pas de where. A implanter */
  if (nbNoeuds==0) {
    (*whereop)=NULL;
    return 0;
  }

  ops=(list_op*)malloc(sizeof(list_op)*nbNoeuds);
  cpt=0;
  whereloc=wherestr;
  
  for (cpt=0;cpt<nbNoeuds;cpt++) {
    /* on sort d'abord le type d'operation */
    whereloc++; /* saut du crochet ouvrant */
    tok=index(whereloc,',');
    tempo=recopien(whereloc,strlen(whereloc)-strlen(tok));
    ops[cpt].type_op=atoi(whereloc);
    if (ops[cpt].type_op!=LOPJOIN) { // fusion des cas 2 et 3 (rpt selection avec chaine et avec numerique)
      ops[cpt].type_op=LOPSELECT;
    }
    free(tempo);
    /* on sort ensuite le type de jointure */
    whereloc=tok+1; /* saut de la virgule */
    tok=index(whereloc,',');
    ops[cpt].op_join=recopien(whereloc,strlen(whereloc)-strlen(tok));
    
    /* on sort enfin les deux operandes */
    whereloc=tok+1; /* saut de la virgule */
    tok=index(whereloc,',');
    attrEtTable(whereloc,&attr,&table);
    ops[cpt].table1=table;
    ops[cpt].at1=attr;
  
    whereloc=tok+1;
    attrEtTable(whereloc,&attr,&table);
    ops[cpt].table2=table;
    ops[cpt].at2=attr;

    /* on va a la fin de l'operation */
    whereloc=index(whereloc,'[');
  }

  (*whereop)=ops;
  return nbNoeuds;
}

void printOperations(list_op* lop,int nbop) {
  int i;

  for (i=0;i<nbop;i++) {
    printf("operation %d\n",i);
    printf("table1 : %s attr1 : %s table2 : %s attr2 : %s op : %s\n",lop[i].table1,lop[i].at1,lop[i].table2,lop[i].at2,lop[i].op_join);
  }
}

void freeOperations(list_op** lop,int nbop){
  int i;
  list_op* local=*lop;
  for (i=0;i<nbop;i++) {
    free(local[i].table1);
    free(local[i].table2);
    free(local[i].at1);
    free(local[i].at2);
    free(local[i].op_join);
  }
  free (local);
  *lop=NULL;
}


int getNbAtt(char* str){
  char* strloc=str;
  int nb;
  if (strlen(str)==0){
    return 0;
  } else {
    nb=1;
    while ((strloc=index(strloc,','))!=NULL) {
      nb++;
      strloc++;
    }
    return nb;
  }
}
/* TODO : il faudra peut etre modifier la structure lsit_attr pour retourner une organisation par table */  
int getAttributes(char* selectstr,list_attr** selectl) {
  int cpt;
  int nbAtt;
  list_attr* la;
  char* attr;
  char* table;
  char* selectloc;

  //printf("SELECT STR %s\n",selectstr);
  nbAtt=getNbAtt(selectstr);
  //printf("NBATT : %d\n",nbAtt);
  if (nbAtt==0) {
    (*selectl)=NULL;
    return 0;
  }

  la=(list_attr*)malloc(sizeof(list_attr)*nbAtt);
  selectloc=selectstr;
  for (cpt=0;cpt<nbAtt;cpt++) {
    attrEtTable(selectloc,&attr,&table);
    la[cpt].table=table;
    la[cpt].at=attr;
    /* faiblesse potentielle : on suppose que la liste des attributs est propre */
    selectloc=index(selectloc,',')+1;
  }

  (*selectl)=la;
  return nbAtt;
}

void printAttributes(list_attr* lat,int nbat) {
  int i;

  for (i=0;i<nbat;i++) {
    printf("projection %d\n",i);
    printf("table : %s attr : %s\n",lat[i].table,lat[i].at);
  }
}

void freeAttributes(list_attr** lat,int nbat) {
  int i;
  list_attr* local=*lat;
  for (i=0;i<nbat;i++) {
    free(local[i].table);
    free(local[i].at);
  }
  free(local);
  *lat=NULL;
}
