#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <math.h>

#include "common.h"
#include "infostables.h"
#include "bloc.h"

/*************************************************************/
/* extraction des infos necessaires (alias) pour la jointure */
/*************************************************************/

/***************************************************************
* Parse l'entete de la table : nom de la table, nom de la base *
* ce deuxieme champ est pour l'instant inutilise               *
***************************************************************/

void parseheader(xmlDocPtr doc,xmlNodePtr curpar,info_table *tab) {
  xmlChar* tuples ; 
  xmlChar* size ;
  xmlNodePtr cur;

  tab->nbtuples = -1 ;
  tab->size = -1 ;
  cur=curpar->xmlChildrenNode;
  while (cur!=NULL) {     
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"tablename"))) {
      tab->nom_table=(char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
    }
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"basename"))) {
      tab->nom_base=(char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
    }
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"tuplesnr"))) {
      tuples = xmlNodeListGetString(doc,cur->xmlChildrenNode,1) ;
      tab->nbtuples=atoi(tuples) ; 
      xmlFree(tuples) ;
    }
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"tablesize"))) {
      size = xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
      tab->size=atoi(size) ;
      xmlFree(size);
    }
    cur=cur->next;
  }
  return;
}

/************************************************************************
* Parse les attributs de la table : nom et type (champ inutilise)       *
************************************************************************/

void parseattributes(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlNodePtr curattribs;
  xmlNodePtr cur;


  table->nbattr=0;
  table->attribs=(info_attr*)malloc(sizeof(info_attr)*NBMAXATTRS);
  
  curattribs=curpar->xmlChildrenNode;
  while (curattribs!=NULL) {     
    if ((!xmlStrcasecmp(curattribs->name,(const xmlChar *)"attribute"))) {
      cur=curattribs->xmlChildrenNode;
      while (cur!=NULL) {     
	if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"attrname"))) {
	  table->attribs[table->nbattr].nom_attr=
	    (char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	}
	if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"attrtype"))) {
	  table->attribs[table->nbattr].type=
	    (char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	}
	cur=cur->next;
      }
      table->nbattr++;
    }
    curattribs=curattribs->next;
  }

  table->attribs=(info_attr*)realloc(table->attribs,sizeof(info_attr)*table->nbattr);
  return;
}

/*********************************************************************
 * Parse les machines sur lesquelles la table est distribuee :       *
 * nom, nb tuples, taille fragment                                   *
 * (seul le nom est utilise pour l'instant)                          * 
 ********************************************************************/
void parsemachines(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlChar *tuples;
  xmlChar *size;

  xmlNodePtr curmachines;
  xmlNodePtr cur;

  table->nbnode = 0 ;
  table->nodes=(info_node*)malloc(sizeof(info_node)*NBMAXNODES);

  curmachines=curpar->xmlChildrenNode;
  while (curmachines!=NULL) {     
    if ((!xmlStrcasecmp(curmachines->name,(const xmlChar *)"machine"))) {
      cur=curmachines->xmlChildrenNode;
      table->nodes[table->nbnode].nbtuples = -1 ;
      table->nodes[table->nbnode].size = -1 ;
      while (cur!=NULL) {
	if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"machinename"))) {
	  table->nodes[table->nbnode].nom_node=(char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	}
	if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"machtuples"))) {
	  tuples=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	  table->nodes[table->nbnode].nbtuples=atoi(tuples);
	  xmlFree(tuples);
	} 
	if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"machsize"))) {
	  size=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	  table->nodes[table->nbnode].size=atoi(size);
	  xmlFree(size);
	}
	cur=cur->next;
      }
      table->nbnode++;
    }
    curmachines=curmachines->next;
  }
  table->nodes=(info_node*)realloc(table->nodes,sizeof(info_node)*table->nbnode);
  return;
}

/*******************************************************************
 * Parse une table : recherche le tag de depart et appelle les     *
 * sous-fonctions ci-dessus                                        *
 ******************************************************************/
void parseTable(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlNodePtr cur;

  cur=curpar->xmlChildrenNode;
  while (cur!=NULL) {     
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"header"))) {
      parseheader(doc,cur,table);
    }
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"attributes"))) {
      parseattributes(doc,cur,table);
    }
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"machines"))) {
      parsemachines(doc,cur,table);
    }
    cur=cur->next;
  }
  return;
}

/*************************************************************
 * Parse le fichier. Appelle parseTable pour chaque table    *
 ************************************************************/
void parseInfo(xmlDocPtr doc,xmlNodePtr cur,info_tables *it) {
  cur=cur->xmlChildrenNode;
  it->nbt=0;
  it->tabs=(info_table*)malloc(sizeof(info_table)*NBMAXTABLES);

  while (cur!=NULL) {
    if ((!xmlStrcasecmp(cur->name,(const xmlChar *)"table"))) {
      parseTable(doc,cur,&(it->tabs[it->nbt]));
      it->nbt++;
    }
    cur=cur->next;
  }

  it->tabs=(info_table*)realloc(it->tabs,sizeof(info_table)*it->nbt);

  return;
}

/******************************************************************
 * Ajout des aliases (id unique pour chaque attribut)             *
 *****************************************************************/
void addAliases(info_tables *it) {
  int numtab;
  int numatt;
  int numalias=0;
  for (numtab=0;numtab<it->nbt;numtab++) {
    for (numatt=0;numatt<it->tabs[numtab].nbattr;numatt++) {
      sprintf(it->tabs[numtab].attribs[numatt].alias_attr,"ATT%03d",numalias);
      numalias++;
    }
    it->tabs[numtab].lastAlias=it->tabs[numtab].attribs[numatt-1].alias_attr;
  }
}
/**********************************************************************
 * recuperation des infos. Parse le fichier xml et ajoute les aliases *
 *********************************************************************/
info_tables* getInfoTables(char* nomFic) {

  xmlDocPtr doc;
  xmlNodePtr cur;
  info_tables *it;

  // Ouverture du fichier
  if (nomFic!=NULL) {
    doc=xmlParseFile(nomFic);
  }else{
    doc=xmlParseFile("info.xml");
  }

  if (doc==NULL) {
    fprintf(stderr,"No xml description of the database\n");
    return NULL;
  }
  
  // Initialisaion du parser
  cur=xmlDocGetRootElement(doc);
  if (cur==NULL) {
    fprintf(stderr,"Empty document\n");
    xmlFreeDoc(doc);
    return NULL;
  } 

  // Alloue la memoire
  it=(info_tables*)malloc(sizeof(info_tables));
  it->nbt=0;
  it->tabs=(info_table*)malloc(sizeof(info_table)*NBMAXTABLES);

  // Parse le fichier
  parseInfo(doc,cur,it);
  // Ajoute les aliases (id) aux attributs
  addAliases(it);

  // Fermeture du fichier
  xmlFreeDoc(doc);

  return it;
}


/*******************************************************************/
char* inTable(info_tables* it, char* table, char* alias) {
  char* restab;
  char* resattr;
  resattr=getAttr(it,&restab,alias);
  if ((resattr!=NULL)&&(strcasecmp(table,restab)==0)) {
    return resattr;
  } else {
    return NULL;
  }
}


void traceInfoNode(info_node* node) {
    printf("Nom: %s", node->nom_node) ;
    if (node->nbtuples > -1) {
      printf(" Nbtuples: %d", node->nbtuples) ;
    }

    if (node->size > -1) {
      printf(" Size: %d", node->size) ;
    }

    printf("\n") ;
}

void traceInfoAttr(info_attr* attr) {
    printf("Nom: %s (alias : %s) Type : %s\n", attr->nom_attr, attr->alias_attr, attr->type);
}

/***********************************************************************/
void traceInfoTable(info_table* tab) {
  int i;

  printf("*****************\n");  
  printf("Nom : %s\n", tab->nom_table);
  printf("Base : %s\n", tab->nom_base);

  if (tab->nbtuples > -1 ) {
    printf("Nbtuples : %d\n", tab->nbtuples);
  }

  if (tab->size > -1 ) {
    printf("Size : %d\n", tab->size);
  }

  for (i=0;i<tab->nbattr;i++) {
    printf("** Attribut #%i : ",i) ;
    traceInfoAttr(&(tab->attribs[i])) ;
  }

  for (i=0;i<tab->nbnode;i++) {
    printf("** Noeud #%i : ",i) ;
    traceInfoNode(&(tab->nodes[i])) ;
  }
  printf("*****************\n");
}

/***********************************************************************/
void traceInfoTables(info_tables* it) {
  int i;
  
  for (i=0;i<it->nbt;i++) {  
    printf("Table #%i :\n",i) ;
    traceInfoTable(&(it->tabs[i])) ;
    printf("\n") ;
  }

}
    
/************************************************************************/
void libereInfoTables(info_tables* it) {
  int i,j;
  for (i=0;i<it->nbt;i++) {
    for (j=0;j<it->tabs[i].nbattr;j++) {
      free(it->tabs[i].attribs[j].nom_attr);
    }
    free(it->tabs[i].nom_table);
  }
  free(it->tabs);
  free(it);
}

/************************************************************************/
void libereInfoTable(info_table* it) {
  int j;
 
  for (j=0;j<it->nbattr;j++) {
    free(it->attribs[j].nom_attr);
  }
  free(it);
}


/*************************************************************/
/* recuperer l'alias correspondant a un attribut d'une table */
/*************************************************************/
char* getAlias(info_tables* it,char* nt, char* na) {
  int i=0,j=0;
  // recherche table
  while ((i<it->nbt) && ((it->tabs[i].used==0)||(strcasecmp(it->tabs[i].nom_table,nt)!=0))) {
    i++;
  }
  if (i==it->nbt) {
    return NULL;
  } else {
    while ((j<it->tabs[i].nbattr)&&(strcasecmp(it->tabs[i].attribs[j].nom_attr,na)!=0)) {
      j++;
    }
    if (j==it->tabs[i].nbattr) {
      return NULL;
    } else {
      return it->tabs[i].attribs[j].alias_attr;
    }
  }
}


/*************************************************************************/
/* cherche si un attribut est defini dans la liste des tables.           */
/*                                                                       */
/* retourne 0 si pas defini, 1 si defini sans ambiguite, >1 si ambiguite */
/*************************************************************************/
int findAttr(info_tables* it,char** nt,char* na) {
  int i,j;
  int found=0;
  *nt=NULL;
  // recherche table
  for (i=0;i<it->nbt;i++) {
    if (it->tabs[i].used) {
      for (j=0;j<it->tabs[i].nbattr;j++) {
	if (strcasecmp(it->tabs[i].attribs[j].nom_attr,na)==0) {
	  found++;
	  *nt=it->tabs[i].nom_table;
	}
      }
    }
  }
  return found;
}


/*************************************************************/
/* recuperer l'attribut correspondant a un alias d'une table */
/*************************************************************/
char* getAttr(info_tables* it,char** nt, char* alias) {
  int i=0;
  int j=0;

  while ((i<it->nbt) && ((it->tabs[i].used==0)||(strcasecmp(it->tabs[i].lastAlias,alias)<0))) {
    i++;
  }
  if (i==it->nbt) {
    *nt=NULL;
    return NULL;
  }
  while ((j<it->tabs[i].nbattr)&&(strcasecmp(it->tabs[i].attribs[j].alias_attr,alias)<0)) {
    j++;
  }
  if (j==it->tabs[i].nbattr) {
    *nt=NULL;
    return NULL;
  } else {
    *nt=it->tabs[i].nom_table;
    return it->tabs[i].attribs[j].nom_attr;
  }
}

/************************************************************************************/
/* marque une table comme etant utilisee et renvoie 1 s'il a trouve la table cherchee */
/************************************************************************************/
int markAsUsed(info_tables* it,char* nt){
  int i=0;
  while ((i<it->nbt) && (strcasecmp(it->tabs[i].nom_table,nt)!=0)) {
    i++;
  }
  if (i<it->nbt) {
    it->tabs[i].used=1;
    return 1;
  } else {
    return 0;
  }
}

/****************************************************************
 * retourne la liste des noeuds de fragmentation de la table nt *
 ***************************************************************/
int getNodes(info_tables* it, char* nt, info_node** nodes){
  int i=0;
  while ((i<it->nbt) && (strcasecmp(it->tabs[i].nom_table,nt)!=0)) {
    i++;
  }
  if (i<it->nbt) {
    (*nodes)=it->tabs[i].nodes;
    return it->tabs[i].nbnode;
  } else {
    (*nodes)=NULL;
    return -1;
  }
}

/*************************************************************/
void markAllUnused(info_tables* it){
  int i;
  for (i=0;i<it->nbt;i++) {
    it->tabs[i].used=0;
  }
}

/********************************************************/
/* trier les attributs d'une table issue d'une jointure */
/********************************************************/
/* en arguments :                                       */
/*   - la liste des attributs de la table issue de la   */
/*     jointure que l'on veut trier                     */
/*   - la liste des attributs de toutes les tables avec */
/*     leurs alias                                      */
/*   - la liste des tables jointes (comme on joint les  */
/*     2 par 2, il y en a 2                             */
/********************************************************/
int TriTableJointe(info_table* it, info_tables* its, char* n1, char* n2){

  int res=1;
  int trouve=0; // pour tester si un attribut du meme nom (mais venant d'une autre table) existe deja
  int nbat=0;
  info_table* temp;
  int i=0;
  int j;
  int k;


  temp->attribs=(info_attr*)malloc(sizeof(info_attr)*NBMAXATTRS);
  
  while ((i<=its->nbt)&&(!trouve)){ // on cherche si la table n1 existe
    if (strcasecmp(its->tabs[i].nom_table,n1)==0) {
      trouve=1;
    }
    else {
      i++;
    }
  }

  if (!trouve) {
    res=0;
  }
  else { // la premiere table existe
    for (j=0; j<its->tabs[i].nbattr; j++){ // on copie les attributs de la premiere table dans l'ordre
      temp->attribs[nbat].nom_attr=strdup(its->tabs[i].attribs[j].nom_attr);
      nbat++;
    }

    trouve=0;
    i=0;

    while ((i<=its->nbt)&&(!trouve)){ // on cherche si la table n2 existe
      if (strcasecmp(its->tabs[i].nom_table,n2)==0) {
	trouve=1;
      }
      else {
	i++;
      }
    }

    if (!trouve) {
      res=0;
    }
    else {
      for (j=0; j<=its->tabs[i].nbattr; j++){ // on copie les attributs de la premiere table dans l'ordre
	trouve=0;
	k=0;
	while ((k<=nbat)&&(!trouve)) {
	  if (strcasecmp(temp->attribs[k].nom_attr,its->tabs[i].attribs[j].nom_attr)==0) { // on regarde si on a deja cet attribut dans la table jointe
	    trouve=1;
	  }
	  else {
	    k++;
	  }
	}

	if (!trouve) { // cet attribut ne se trouve pas encore dans la table jointe
	  temp->attribs[nbat].nom_attr=strdup(its->tabs[i].attribs[j].nom_attr);
	  nbat++;
	}
      }
      
      // si les 2 tables existent, on a gnr la liste des attributs dans le bon ordre...
      libereInfoTable(it);
      temp->attribs=(info_attr*)realloc(temp->attribs,sizeof(info_attr)*nbat);
      
      it=temp;

    }

  
  }
  return res;

}


/*****************************************************/
/* Creer une liste vide de couples (attribut, alias) */
/*****************************************************/
attr_list* CreerListeVide (){
  return NULL;
}


/**********************************************************/
/* Ajout d'un couple (attribut, alias) en fin de la liste */
/**********************************************************/
void AddInfoQueue (attr_list* L, char* at, char* al){
  attr_list* new;
  attr_list* last;
  int vide=1;
  int i;

  new=(attr_list*)malloc(sizeof(attr_list));

  new->attrib.nom_attr=strdup(at);
  
  //  new->attrib.alias_attr=(char*)malloc(7*sizeof(char));

  for (i=0; i<=6; i++) {
    new->attrib.alias_attr[i]=al[i];
  }
  new->suivant=NULL;

  last=L;

  if (last != NULL) {
    vide=0;
    while (last->suivant != NULL) {
      last=last->suivant;
    }
  }

  if (vide) {
    L=new;
  }
  else {
    last->suivant=new;
  }

}


/*************************************************/
/* Test si la liste est triee, si non on la trie */
/*************************************************/
/* L1 = liste bien triee, L2 = liste dont on     */
/* verifie si bien trie                          */
/* renvoie 1 si L2 ressort etait triee,          */
/*         0 si L2 intriable (nb d'elt different */
/*           ou alias manquants                  */
/*************************************************/
int TriParAlias (attr_list* L1, attr_list* L2){

  attr_list* cL1 = L1;
  attr_list* cL2 = L2;
  attr_list* prec;
  attr_list* cour;

  attr_list* res = CreerListeVide ();
  attr_list* last;

  //  int trie = 1;    // pour tester si L2 est trie
  int triable = 1; // pour tester si on peut trier L2 (cad que L2 contient tous les alias de L1, et pas plus ou moins)
  int mmnb = 1;    // pour tester si les 2 listes ont le meme nb d'elements
  int trouve = 0;  // pour tester si on trouve chaque alias de L1 dans L2

  // i. on verifie que les 2 listes ont le meme nb d'elements
  while (cL1 != NULL) {
    if (cL2 != NULL) {
      cL1 = cL1->suivant;
      cL2 = cL2->suivant;
    }
    else { // L2 a moins d'elements que L1
      mmnb = 0;
    }
  }

  if  (cL2 != NULL) { // L2 a plus d'elements que L1
    mmnb = 0;
  }


  // ii. on verifie que tous les alias des elements de L1 sont presents dans L2

  if (mmnb == 1) { // si les 2 listes comptent le meme nb d'elements
    cL1 = L1;
    while ((cL1 != NULL) && triable) {
      cL2 = L2;
      while ((cL2 != NULL) && !trouve) {
	if (strcasecmp(cL1->attrib.alias_attr, cL2->attrib.alias_attr) == 0) { // on compare chaque element de L2 avec ceux de L1
	  trouve = 1;
	}
	else {
	  cL2 = cL2->suivant;
	}
      }
      cL1 = cL1->suivant;
      if (!trouve) {
	triable = 0;
      }
      trouve = 0;
    }
  }


  // iii. on trie L2 par rapport a L1 

  if ((mmnb == 1) && (triable == 1)) { // il y a autant d'elt ds chaque liste et chaque alias de L1 existe ds L2
    cL1 = L1;
    cL2 = L2;
    while (cL1 != NULL) {
      prec = cL2;
      cour = prec->suivant;

      if (prec->attrib.alias_attr == cL1->attrib.alias_attr) { // si le premier element est celui que l'on cherche
	if (last == NULL) { // la liste triee ne contient pas encore d'element
	  res = prec;
	  last = prec;
	  cL2 = cL2->suivant;
	  last->suivant = NULL;
	}
	else {
	  last->suivant = prec;
	  cL2 = cL2->suivant;
	  last = last->suivant;
	  last->suivant = NULL;
	}
      }

      else {
	while ((cour != NULL) && (cour->attrib.alias_attr != cL1->attrib.alias_attr)) { // on avance jusqu'a ce que cour pointe sur l'element cherche
	  prec = cour;
	  cour = prec->suivant;
	}

	if (last == NULL) { // la liste triee ne contient pas encore d'element
	  res = cour;
	  last = cour;
	  prec->suivant = cour->suivant;
	  last->suivant = NULL;
	}
	else {
	  last->suivant = cour;
	  prec->suivant = cour->suivant;
	  last = last->suivant;
	  last->suivant = NULL;
	}
      }
    
      cL1 = cL1->suivant;
    }
  }
  
  
  // iv. on retourne le resultat
  
  return (mmnb+triable - mmnb*triable); // il faut que mmnb = 1 et triable = 1 pour que la liste ressorte triee
  
}

/*************************************************
 * Creation d'un nouvel attribut                 *
 *************************************************/
info_attr* createAttr(char* nom_attr, char* alias_attr, char* type) {
  info_attr* unAttr = (info_attr*)malloc(sizeof(info_attr)) ;
  unAttr->nom_attr = nom_attr ;
  strcpy(unAttr->alias_attr, alias_attr) ; 
  unAttr->type = type ;
}  

/*************************************************
 * Creation d'une nouvelle machine               *
 * mettre -1 aux paramtres nbtuples ou size     *
 * si ils ne sont pas utilises                  *
 *************************************************/
info_node* createNode(char* nom_node, int nbtuples, int size) {
  info_node* unNode = (info_node*)malloc(sizeof(info_node)) ;
  unNode->nom_node = nom_node ;
  unNode->nbtuples = nbtuples ;
  unNode->size = size ;
}

/*************************************************
 * Creation d'une nouvelle table                 *
 * mettre -1 aux paramtres nbtuples ou size     *
 * si ils ne sont pas utilises                  *
 *************************************************/
info_table* createTable(char* nom_table, char* nom_base, char* lastAlias,
			int used, int nbtuples, int size,
			int nbattr, info_attr* attribs, int nbnode, info_node* nodes
			) {

  info_table* tab = (info_table*)malloc(sizeof(info_table)) ;
  tab->nom_table = nom_table ;
  tab->nom_base = nom_base ;
  tab->nbattr = nbattr ;
  tab->lastAlias = lastAlias ;
  tab->attribs = attribs ;
  tab->nbnode = nbnode ;
  tab->nodes = nodes ;
  tab->used = used ;
  tab->nbtuples = nbtuples ;
  tab->size = size ;
  
  return tab ;
}

/*************************************************
 * Ajout d'une table  une liste de tables       *
 *************************************************/
int addTable(info_tables* it, info_table* tab) {
  if (it->nbt == NBMAXTABLES) {
    printf("Can't add another table, there are already %d tables in this database",NBMAXTABLES) ;
    return 0 ;
  }
  else {
    it->tabs=(info_table*)realloc(it->tabs,sizeof(info_table)*++(it->nbt));  
    it->tabs[it->nbt - 1] = *tab ;
    return 1 ;
  }
}

/*************************************************
 * Ajout d'un attribut  une table               *
 *************************************************/
int addAttr(info_table* tab, info_attr* aAttr) {
   if (tab->nbattr == NBMAXATTRS) {
    printf("Can't add another attr, there are already %d attrs in this table",NBMAXATTRS) ;
    return 0 ;
  }
  else {
    tab->attribs=(info_attr*)realloc(tab->attribs,sizeof(info_attr)*++(tab->nbattr));  
    tab->attribs[tab->nbattr - 1] = *aAttr ;
    return 1 ;
  }
}

/*************************************************
 * Ajout d'une machine  une table               *
 *************************************************/
int addNode(info_table* tab, info_node* aNode) {
   if (tab->nbnode == NBMAXNODES) {
    printf("Can't add another node, there are already %d nodes in this table",NBMAXNODES) ;
    return 0 ;
  }
  else {
    tab->nodes=(info_node*)realloc(tab->nodes,sizeof(info_node)*++(tab->nbnode));  
    tab->nodes[tab->nbnode - 1] = *aNode ;
    return 1 ;
  }
}

/**************************************************
 * Fonction de ecriture d'un fichier xml a partir *
 * d'une structure info_table                     *
 **************************************************/
int writeInfoTables(char* fileName, info_tables* it) {
  xmlDocPtr doc ;
  xmlNodePtr cur ;
  int i,j,p ;
  
  xmlKeepBlanksDefault(0) ;
  doc = xmlNewDoc(NULL) ;
  xmlDocSetRootElement(doc, xmlNewDocNode(doc, NULL, "config", NULL)) ;
  
  /* Boucle sur les tables */

  for (j = 0 ; j < it->nbt ; j++) {
    info_table* uneTab = &(it->tabs[j]) ;
    /* creation du header */
    cur = xmlDocGetRootElement(doc) ; 
    cur = xmlNewTextChild(cur, NULL, "table", NULL) ;
    cur = xmlNewTextChild(cur, NULL, "header", NULL) ;
    xmlNewTextChild(cur, NULL, "tablename", uneTab->nom_table) ;
    xmlNewTextChild(cur, NULL, "basename", uneTab->nom_base) ;
    
    if (uneTab->nbtuples != -1) {
      char * tuples;
      sprintf(tuples,"%d",uneTab->nbtuples) ;
      xmlNewTextChild(cur, NULL, "tuplesnr", tuples) ;
    }
    
    if (uneTab->size != -1) {
      char * size;
      sprintf(size,"%d",uneTab->size) ;
      xmlNewTextChild(cur, NULL, "tablesize", size) ;
    }
    
    /*********************/
    
    /* creation des attributs */
    cur = xmlNewTextChild(cur->parent, NULL, "attributes", NULL) ;
    
    for(i = 0 ; i < uneTab->nbattr ; i++) {
      xmlNodePtr curAttribut = xmlNewTextChild(cur, NULL, "attribute", NULL) ;
      info_attr* attr = &(uneTab->attribs[i]) ;
      xmlNewTextChild(curAttribut, NULL, "attrname", attr->nom_attr) ;
      xmlNewTextChild(curAttribut, NULL, "attrtype", attr->type) ;
    }
    /*************************/
    
    /* creation des machines */
    
    cur = xmlNewTextChild(cur->parent, NULL, "machines", NULL) ;
    
    for(i = 0 ; i < uneTab->nbnode ; i++) {
      xmlNodePtr curNode = xmlNewTextChild(cur, NULL, "machine", NULL) ;
      info_node* node = &(uneTab->nodes[i]) ;
      xmlNewTextChild(curNode, NULL, "machinename", node->nom_node) ;
      
      if (node->nbtuples != -1) {
	p = 0 ;
	// on determine la taille de la chaine de caractere
	// correspond a l'entier node->nbtuples
	while (((node->nbtuples)/((int)pow(10,p++))) != 0 ) ;
	char * tuples = (char*)malloc(sizeof(char)*p) ;
	sprintf(tuples, "%d", node->nbtuples) ;
	xmlNewTextChild(curNode, NULL, "machtuples", tuples) ;
      }
      
      if (node->size != -1) {
	p = 0 ;
	// on determine la taille de la chaine de caractere
	// correspond a l'entier node->size
	while (((node->size)/((int)pow(10,p++))) != 0 ) ;
	char * size = (char*)malloc(sizeof(char)*p) ;
	sprintf(size, "%d", node->size) ;
	xmlNewTextChild(curNode, NULL, "machsize", size) ;
      }
      
    }
    
    /*************************/
  }
  /*fin de la boucle sur les tables */

  /* ecriture du fichier */
  xmlSaveFormatFile(fileName, doc, 1) ;
  /***********************/

  xmlFree(cur) ;
  xmlFreeDoc(doc) ;
  return 1;
}


/* main(){ */
/*   info_tables* it; */
/*   info_table* uneTab ; */
  
/*   uneTab = createTable("test", "metin", "ATT000", 0, 0, 0, 0, NULL, 0, NULL) ; */
/*   addAttr(uneTab, createAttr("attribut", "ATT000", "TEXT")) ; */
/*   addNode(uneTab, createNode("grappe12", 0, 0)) ; */

/*   it = getInfoTables("info.xml") ; */
/*   addTable(it, uneTab) ; */

/*   traceInfoTables(it); */
/*   writeInfoTables("test.xml", it) ; */
/*   exit(1) ; */
/*   //  free(uneTab) ; */
/*   //libereInfoTables(it);  */

/*   /\* */
/*   traceInfoTable(uneTab) ; */
/*   addTable(it, uneTab) ; */
/*   writeInfoTable("info.xml", uneTab) ; */
/*   //  traceInfoTables(it); */
/*   libereInfoTable(uneTab) ; */
/*   libereInfoTables(it); */
/*   // char *table=recopie("A"); */
/*   // char *attr=recopie("nom"); */
/*   // char *alias=recopie("ATT01"); */
/*   // int i,j; */


  
/*   // printf("Attribut : %s Alias %s\n",it->tabs[0].attribs[0].nom_attr,it->tabs[0].attribs[0].alias_attr); */
/*   // printf("%i tables\n",it->nbt); */
/*   // printf("Attribut : %s Alias %s\n",it->tabs[0].attribs[0].nom_attr,it->tabs[0].attribs[0].alias_attr); */




/* /\*   for (i=0;i<1;i++) { *\/ */
/* /\*     it[i]=getInfoTables(NULL); *\/ */
/* /\*     //printf("alias de %s.%s : %s\n",table,attr,getAlias(it[i],table,attr)); *\/ */
/* /\*   } *\/ */
/* /\*   for (i=0;i<1;i++) { *\/ */
/* /\*     for (j=0;j<1000;j++) { *\/ */
/* /\*       getAlias(it[i],table,attr); *\/ */
/* /\*       //printf("alias de %s.%s : %s\n",table,attr,getAlias(it[i],table,attr)); *\/ */
/* /\*       inTable(it[i],table,alias); *\/ */
/* /\*       //printf("inTable de %s.%s : %s\n",table,alias,inTable(it[i],table,alias)); *\/ */
/* /\*     } *\/ */
/* /\*   } *\/ */
/* /\*   for (i=0;i<1;i++) { *\/ */
/* /\*     traceInfoTables(it[i]); *\/ */
/* /\*   } *\/ */
  
/* /\*   for (i=0;i<1;i++) { *\/ */
/* /\*     libereInfoTables(it[i]); *\/ */
/* /\*   } *\/ */

/* } */

