#ifndef INFOSTABLESH

// #define TAILLELIGNE 500 // inutile dans la version ou info.db est un fichier binaire
#define NBMAXTABLES 30
#define NBMAXATTRS 30
#define NBMAXNODES 30

typedef struct _info_attr {
  char* nom_attr;
  char alias_attr[7]; /* "ATT" + numero (de 0 a 999), chaque attribut a un alias unique (cad une cle) */
  char* type; /* Type de l'attribut (TEXT,NUM,...) */
} info_attr;

typedef struct _info_node {
  char* nom_node; // le nom de la node
  int nbtuples;    // nombre de tuples sur la node -1 si c'est pas utilis
  int size;// a commenter sur la node -1 si c'est pas utilis
} info_node;

// structure de description d'une table
typedef struct _info_table {
  char* nom_table; // le nom de la table
  char* nom_base;  // la base dans laquelle se trouve la table
  int nbattr;      // le nombre d'attributs de la table
  char* lastAlias; // l'alias du dernier attribut (permet un recherche rapide sur les alias)
  info_attr* attribs; // liste des attributs (+leur alias)
  int nbnode;      // le nombre de nodes sur lesquelles la table est repartie
  info_node* nodes;    // la liste de ces nodes
  int used;        // indique si la table est utilisee (a reexpliquer)
  int nbtuples;    // nombre de tuples dans la table sur la node -1 si c'est pas utilis
  int size;// a commenter sur la node -1 si c'est pas utilis
} info_table;

// structure pour la liste des tables
typedef struct _info_tables {
  int nbt;
  info_table* tabs;
} info_tables;

// structure pour la liste des attributs (liste chainee. utilisee ? nicolas?)
typedef struct _attr_list {
  info_attr attrib;
  struct _attr_list* suivant;
} attr_list;

info_tables* getInfoTables(char* nomFic);

void libereInfoTable(info_table* it);
void libereInfoTables(info_tables* it);
char* getAlias(info_tables* it,char* nt, char* na);
char* getAttr(info_tables* it,char** nt, char* alias);
char* inTable(info_tables* it, char* table, char* alias);
int findAttr(info_tables* it,char** nt,char* na);
int markAsUsed(info_tables* it,char* nt);
void markAllUnused(info_tables* it);
int getNodes(info_tables* it, char* nt,info_node** nodes) ;

/** fonctions relatives a des attr_list **/
attr_list* CreerListeVide ();
void AddInfoQueue (attr_list* L, char* at, char* al);
int TriParAlias (attr_list* L1, attr_list* L2);

/** Fonctions pour l'ajout et l'ecriture de tables **/
info_table* createTable(char* nom_table, char* nom_base, char* lastAlias,
			int used, int nbtuples, int size,
			int nbattr, info_attr* attribs, int nbnode, info_node* nodes
			) ;
info_attr* createAttr(char* nom_attr, char* alias_attr, char* type) ;
info_node* createNode(char* nom_node, int nbtuples, int size) ;
int addTable(info_tables* it, info_table* tab) ;
int addAttr(info_table* tab, info_attr* aAttr) ;
int addNode(info_table* tab, info_node* aNode) ;
int writeInfoTables(char* fileName, info_tables* it) ;

/** Fonctions d'affichage **/
void traceInfoNode(info_node* node) ;
void traceInfoAttr(info_attr* attr) ;
void traceInfoTable(info_table* tab) ;
void traceInfoTables(info_tables* it) ;

//info_table* createTable(char* nom_table, char* nom_base) ;

#define INFOSTABLESH
#endif
