#include "parser.h"
#include "common.h"
#include "decompose.h"
#include "treeconst.h"

#define NBMAXATR 20

typedef struct _list_tables{
  int nbt;
  char* tables[100];
} list_tables;

/* initialisation de tu */
void initTU(list_tables* tu){
  tu->nbt=0;
}

/* ajout d'une table dans tu */
void addTU(list_tables* tu,char *table){
  tu->tables[tu->nbt++]=table;
}

/* indique le rang d'une table dans tu si elle s'y trouve, -1 sinon */ 
int rangTable(list_tables* tu,char *table){
  int cpt=0;
  while ((cpt<tu->nbt) && (strcasecmp(tu->tables[cpt],table)!=0)) {
    cpt++;
  }
  if (cpt==tu->nbt) {
    return -1;
  } else {
    return cpt;
  }
}

/* ajoute les tables de op a la liste des tables deja referencees dans l'arbre (si elles n'y figurent pas deja */
void ajoutTU(list_tables* tu, list_op op) {
  if (rangTable(tu,op.table1)==-1) {
    addTU(tu,op.table1);
  }
  if ((op.table2!=NULL)&&(rangTable(tu,op.table2)==-1)) {
    addTU(tu,op.table2);
  }
}

/* retourne le rang de la premiere operation de ops qui soit inserable dans l'arbre,
   c'est a dire dont une des tables fasse partie des tables deja referencees dans l'arbre */
int rangInserable(list_op* ops,int* util,list_tables* tu,int nbN,int* nt) {
  int cpt=0;
  while (cpt<nbN) {
    if (util[cpt]==0) {
      if (rangTable(tu,ops[cpt].table1)!=-1) {
	(*nt)=1;
	return cpt;
      } else  if ((ops[cpt].table2!=NULL)&&(rangTable(tu,ops[cpt].table2)!=-1)) {
	(*nt)=2;
	return cpt;
      }
    } else {
      cpt++;
    }
  }
  return -1;
}

/****** Met une operation elementaire au format sql2 ****/
list_sql2 *op2sql2(list_op where,int type,list_sql2* prec,int nt,int numres,info_tables* it) {
  int i; // compteur pour la liste des noeuds
  info_node* nodes1; // liste des noeuds de table1 
  info_node* nodes2; // liste des noeuds de table2

  list_sql2* op=(list_sql2*)malloc(sizeof(list_sql2));
  
  /* on sort d'abord le type d'operation */
  if (where.table2==NULL) {
    op->type=OP_SELECT;
  } else {
    op->type=OP_JOIN;
  }
  /* on sort ensuite le type de jointure */
  op->op_join=recopie(where.op_join);
  /* on sort enfin les deux operandes */
  /* la table 1 est en fait un resultat intermediaire */
  if (nt&1) {
    op->table1=recopie(prec->tableres);
    op->at1=recopie(getAlias(it,where.table1,where.at1));
    // La liste des noeuds ??? cf TODO plus bas
    op->nbnodes1=0;
    op->nodes1=NULL;
  } else {
    op->table1=recopie(where.table1);
    //printf("TABLE1 %s\n",op->table1);
    op->at1=recopie(where.at1);
    // recup du nombre de noeud et de la liste des noeuds (noms)
    op->nbnodes1=getNodes(it,where.table1,&nodes1);
    op->nodes1=(int*)malloc(sizeof(int)*op->nbnodes1);
    // conversion des noms en rangs mpi. pourrait etre fait des le chargement des infostables...
    for (i=0;i<op->nbnodes1;i++) {
      op->nodes1[i]=getNodeRank(nodes1[i].nom_node);
    }
  }
  /* la table 2 est en fait un resultat intermediaire */
  if (nt&2) {
    op->table2=recopie(prec->tableres);
    //printf("getAlias(%s,%s)=%s\n",where.table2,where.at2,getAlias(it,where.table2,where.at2));
    op->at2=recopie(getAlias(it,where.table2,where.at2));
    // La liste des noeuds ???*
    // TODO : si tempo, alors on prend les noeuds du res. Ou les trouver ???
    op->nbnodes2=0;
    op->nodes2=NULL;
  } else {
    op->table2=recopie(where.table2);
    op->at2=recopie(where.at2);
    // recup du nombre de noeud et de la liste des noeuds (noms)
    op->nbnodes2=getNodes(it,where.table2,&nodes2);
    op->nodes2=(int*)malloc(sizeof(int)*op->nbnodes2);
    // conversion des noms en rangs mpi. pourrait etre fait des le chargement des infostables...
    for (i=0;i<op->nbnodes2;i++) {
      op->nodes2[i]=getNodeRank(nodes2[i].nom_node);
    }  
  }
  /* on indique les tables intermediaires */
  op->todelete=nt;

  /* on initialise le reste */
  op->at1_res=NULL;
  op->at2_res=NULL;
  op->nb_at1_res=0;
  op->nb_at2_res=0;
  op->tableres=(char*)malloc(sizeof(char)*5); // permet 99 tables !
  sprintf(op->tableres,"TR%d",numres);
  op->machine=NULL;
  op->requete_sql=NULL;
  op->suivant1=prec;
  op->suivant2=NULL;
}

/**********************************************************************************
retourne la liste et le nombre de noeuds d'une table (temporaire)
cherche dans la liste chainee d'instructions de type list_sql2 enracinee en "tete",
la presence de la table "table".
Si on la trouve, on recopie les noeuds sur lesquels elle est distribuee
dans "nodes" et on retourne le nombre de noeuds.
Sinon, on met "nodes" a NULL et on retourne -1.
 **********************************************************************************/

int getNodesTableRes(list_sql2* tete,char* table,int** nodes) {
  //  printf("recherche de %s : %s\n",table,tete->tableres);
  if (strcasecmp(tete->tableres,table)==0) {
    (*nodes)=(int*)malloc(sizeof(int)*tete->nbnodesD);
    memcpy((*nodes),tete->nodesD,sizeof(int)*tete->nbnodesD);
     return tete->nbnodesD;
  }
  if (tete->suivant1) { // pas a ce niveau, appel recursif
    return getNodesTableRes(tete->suivant1,table,nodes);
  } else { //erreur : la table tempo est introuvable en amont de l'arbre d'exec
    printf("ERREUR Table %s non trouvee!!!!\n",table);
    (*nodes)=NULL;
    return -1;
  }
  /*  if (strcasecmp(tete->table1,table)==0) {
    (*nodes)=(int*)malloc(sizeof(int)*tete->nbnodes1);
    memcpy((*nodes),tete->nodes1,sizeof(int)*tete->nbnodes1);
     return tete->nbnodes1;
  }
  if (strcasecmp(tete->table2,table)==0) {
    (*nodes)=(int*)malloc(sizeof(int)*tete->nbnodes2);
    memcpy((*nodes),tete->nodes2,sizeof(int)*tete->nbnodes2);
    return tete->nbnodes2;
  }
  if (tete->suivant1) { // pas a ce niveau, appel recursif
    return getNodesTableRes(tete->suivant1,table,nodes);
  } else { //erreur : la table tempo est introuvable en amont de l'arbre d'exec
    (*nodes)=NULL;
    return -1;
  }
  */
}

/**********************************************************************************
 Ajout des noeuds pour les resultats intermediaires.
 Dans op2sql2 on se contente de mettre les noeuds des relations sources.
ici on choisit les noeuds des resultats (embryon d'optimisation en somme...)
 **********************************************************************************/
void ajoutNodes(list_sql2* tete) {
  int i;
  // appel recursif
  if (tete->suivant1) {
    ajoutNodes(tete->suivant1);
    // pour toutes les operations sauf la derniere on regarde si on a des tables qui sont des resultats intermediaires
    // et on cree nodes et nbnodes pour cette table en fonction du resultat precedent (a verifier, a-t-on toujours le bon enchainement ???)
    if (tete->nbnodes1==0) { // table1 est temporaire
      // on recherche son origine et on reprend les sites de l'origine
      tete->nbnodes1=getNodesTableRes(tete->suivant1,tete->table1,&(tete->nodes1));
    }
    if (tete->nbnodes2==0) { // table2 est temporaire
      // on recherche son origine et on reprend les sites de l'origine
      tete->nbnodes2=getNodesTableRes(tete->suivant1,tete->table2,&(tete->nodes2));
    }
  }

  // on choisit toujours les noeuds de la table1
  tete->nbnodesD=tete->nbnodes1;
  tete->nodesD=(int*)malloc(sizeof(int)*tete->nbnodesD);
  memcpy(tete->nodesD,tete->nodes1,sizeof(int)*tete->nbnodesD);
  /*printf("table %s ndnodesD %d nbnodes1 %d nbnodes2 %d\n",tete->tableres,tete->nbnodesD,tete->nbnodes1,tete->nbnodes2);
  printf("nodes1\n");
  for (i=0;i<tete->nbnodes1;i++)
    printf("%d\n",tete->nodes1[i]);
  printf("nodes2\n");
  for (i=0;i<tete->nbnodes2;i++)
    printf("%d\n",tete->nodes2[i]);  
  printf("nodesD\n");
  for (i=0;i<tete->nbnodesD;i++)
    printf("%d\n",tete->nodesD[i]);
  */
}

// TODO : quand les projections sont faites, il faut revoir la projection de la tete pour redonner des noms potables aux attributs...
void ajoutProj(info_tables* it,list_sql2* tete,list_attr* select,int nbs) {
  int nbal;
  int i;
  int nba1;
  int nba2;
  int nbtt=0;
  list_sql2* curr;
  char* listAlias[200];
  char* attr;
  int premier=1;
  char* tempotab;
  char* tempoattr;

  for (nbal=0;nbal<nbs;nbal++) {
    listAlias[nbal]=recopie(getAlias(it,select[nbal].table,select[nbal].at));
    //printf("listAlias[%d] : %s pour %s.%s\n",nbal, listAlias[nbal],select[nbal].table,select[nbal].at);
  }
  for (i=nbal;i<200;i++) {
    listAlias[i]=NULL;
  }

  curr=tete;

  while (curr!=NULL) {
    /** on donne le nom de la table projetee **/
    /*
      curr->tableres=(char*)malloc(sizeof(char)*5);
    sprintf(curr->tableres,"TR%d",nbtt);
    */
    /* on gre la projection */
    curr->at1_res=(char**)malloc(sizeof(char*)*NBMAXATR);
    nba1=0;
    curr->at2_res=(char**)malloc(sizeof(char*)*NBMAXATR);
    nba2=0;    /* Le cas de deux tables temporaires. Pas gere pour l'instant */
    if (curr->todelete==3) {
      printf("erreur todelete\n");
      return;
    }
    // on aura des alias sur la table1 si c'est l'operation finale
    // ou si c'est une table source
    if ((premier) || (((curr->todelete)&1)==0)) {
      curr->al1_res=(char**)malloc(sizeof(char*)*NBMAXATR);
    } else {
      curr->al1_res=NULL;
    }
    // on aura des alias sur la table2 (si c'est l'operation finale
    // ou si c'est une table source) ET si l'op n'est pas une selection :-)
    if ( ((premier) || (((curr->todelete)&2)==0)) && (curr->table2!=NULL)) {
      curr->al2_res=(char**)malloc(sizeof(char*)*NBMAXATR);
    }else{
      curr->al2_res=NULL;
    }

    /** On commence par enlever les attributs qui proviennent de la ou des tables sources introduites dans cette operation **/
    if (curr->todelete!=1) {
      /* les attributs extraits ici ne doivent plus tre propags, d'ou le NULL */
      for (i=0;i<nbal;i++) {
	//printf("inTable(%s,%s)=%s\n",curr->table1,listAlias[i],inTable(it,curr->table1,listAlias[i]));
	if ((listAlias[i]!=NULL)&&((attr=inTable(it,curr->table1,listAlias[i]))!=NULL)) {
	  curr->at1_res[nba1]=recopie(attr);
	  if (premier) {//le traitement est modifie pour l'operation de tete. on ne produit pas les aliases standards, mais un alias "nomtable_nomattr"
	    curr->al1_res[nba1]=(char*)malloc(strlen(curr->table1)+strlen(attr)+2);
	    sprintf(curr->al1_res[nba1],"%s%c%s",curr->table1,SEP_CHAR,attr);
	  }else{
	    curr->al1_res[nba1]=recopie(listAlias[i]);
	  }
	  listAlias[i]=NULL;
	  nba1++;
	}
      }
    }
    if ((curr->table2!=NULL)&&(curr->todelete!=2)) {
      //printf("debut2\n");
      /* les attributs extraits ici ne doivent plus tre propags, d'ou le NULL */
      for (i=0;i<nbal;i++) {
	if ((listAlias[i]!=NULL)&&((attr=inTable(it,curr->table2,listAlias[i]))!=NULL)) {
	  curr->at2_res[nba2]=recopie(attr);
	  if (premier) {//le traitement est modifie pour l'operation de tete. on ne produit pas les aliases standards, mais un alias "nomtable_nomattr"
	    curr->al2_res[nba2]=(char*)malloc(strlen(curr->table2)+strlen(attr)+2);
	    sprintf(curr->al2_res[nba2],"%s%c%s",curr->table2,SEP_CHAR,attr);
	  }else{
	    curr->al2_res[nba2]=recopie(listAlias[i]);
	  }
	  listAlias[i]=NULL;
	  nba2++;
	}
      }
      //printf("fin2\n");
    }
    /* on reporte sur la table temporaire tous les alias restants */
    if (curr->todelete==1) {
      for (i=0;i<nbal;i++) {
	if (listAlias[i]!=NULL) {
	  curr->at1_res[nba1]=recopie(listAlias[i]);
	  if (premier) {
	    tempoattr=getAttr(it,&tempotab,listAlias[i]);
	    curr->al1_res[nba1]=(char*)malloc(strlen(tempoattr)+strlen(tempotab)+2);
	    sprintf(curr->al1_res[nba1],"%s%c%s",tempotab,SEP_CHAR,tempoattr);
	  }
	  nba1++;
	}
      }
    } else {
      for (i=0;i<nbal;i++) {
	if (listAlias[i]!=NULL) {
	  curr->at2_res[nba2]=recopie(listAlias[i]);
	  if (premier) {
	    tempoattr=getAttr(it,&tempotab,listAlias[i]);
	    curr->al2_res[nba2]=(char*)malloc(strlen(tempoattr)+strlen(tempotab)+2);
	    sprintf(curr->al2_res[nba2],"%s%c%s",tempotab,SEP_CHAR,tempoattr);
	  } else {
	  }
	  nba2++;
	}    
      }
    }

    /** on redimensionne les tableaux d'attributs au plus juste
     et on met a jour les champs nb_at1_res et nb_at2_res **/
    curr->at1_res=(char**)realloc(curr->at1_res,sizeof(char*)*nba1);
    if (curr->al1_res!=NULL) {
      curr->al1_res=(char**)realloc(curr->al1_res,sizeof(char*)*nba1);
    }
    curr->nb_at1_res=nba1;
    curr->at2_res=(char**)realloc(curr->at2_res,sizeof(char*)*nba2);
    if (curr->al2_res!=NULL) {
      curr->al2_res=(char**)realloc(curr->al2_res,sizeof(char*)*nba2);
    }
    curr->nb_at2_res=nba2;  

    /** on ajoute a la liste des alias les attributs impliques dans l'operation,
     sachant qu'ils sont deja representes par un alias **/
    if (curr->todelete&1) {
      listAlias[nbal]=curr->at1;
      nbal++;
    }
    if (curr->todelete&2) {
      listAlias[nbal]=curr->at2;
      nbal++;
    }

premier=0;
curr=curr->suivant1;
  }
}

void set_sql2_queries(list_sql2* tete) {
  list_sql2* op=tete;
  int i;
  char * txtquery=(char*)malloc(1024);

  while (op!=NULL) {
    strcpy(txtquery,"CREATE TABLE ");
    strcat(txtquery,op->tableres);
    strcat(txtquery," AS SELECT ");
    for (i=0;i<op->nb_at1_res;i++) {
      strcat(txtquery,op->at1_res[i]);
      if (op->al1_res!=NULL) {
	strcat(txtquery," AS ");
	strcat(txtquery,op->al1_res[i]);
      }
      strcat(txtquery,",");
    }
    for (i=0;i<op->nb_at2_res;i++) {
      strcat(txtquery,op->at2_res[i]);
      if (op->al2_res!=NULL) {
	strcat(txtquery," AS ");
	strcat(txtquery,op->al2_res[i]);
      }
      strcat(txtquery,",");
    }
    // on supprime la derniere virgule
    if ((op->nb_at1_res+op->nb_at1_res)>0) {
      txtquery[strlen(txtquery)-1]=0;
    }
    strcat(txtquery," FROM ");
    strcat(txtquery,op->table1);
    if (op->table2!=NULL) {
      strcat(txtquery,",");
      strcat(txtquery,op->table2);
    }
    strcat(txtquery," WHERE ");
    strcat(txtquery,op->at1);
    strcat(txtquery,op->op_join);
    strcat(txtquery,op->at2);
    
    op->requete_sql=recopie(txtquery);
    op=op->suivant1;
  }
}


void controleTables (info_tables* it,char* tabs) {
  char* tok;
  char* loc=recopie(tabs);
  char* debut=loc;

  if (strlen(tabs)==0) {
    printf("Erreur : pas de clause FROM\n");
    exit(0);
  }
  do {
    tok=index(loc,',');
    if (tok!=NULL) {
      tok[0]=0; // place en fait un '/0' dans loc..
    } 
    if (markAsUsed(it,loc)==0) {
      printf("Erreur : table %s inexistante\n",loc);
      exit(0);
    } else {
      // printf("table %s marquee\n",loc);
    }
    loc=loc+strlen(loc)+1; // ne doit pas etre dangereux, meme a la fin
  } while (tok!=NULL);
  free(debut);
}

void controleOps(info_tables* it,list_op* where,int nbop) {
  int i;
  int trouve;
  char* nt;
  for (i=0;i<nbop;i++) {
    if (where[i].table1==NULL) { // la table n'est pas definie, on cherche a completer
      trouve=findAttr(it,&nt,where[i].at1);
      switch (trouve) {
      case 0 : printf("Erreur dans la requete : attribut %s inconnu\n",where[i].at1);
	exit(0);
      case 1 : where[i].table1=recopie(nt);
	break;
      default : printf("Erreur dans la requete : attribut %s ambigu\n",where[i].at1);
	exit(0);
      }
    } else { //la table est definie, on verifie qu'elle possede l'attribut indique
      if (getAlias(it,where[i].table1,where[i].at1)==NULL) {
	printf("Erreur dans la requete : l'attribut %s n'est pas defini pour la table %s\n",where[i].at1,where[i].table1);
	exit(0);
      }
    }
    if (where[i].type_op==LOPJOIN) {// la partie droite designe un attribut
      if (where[i].table2==NULL) {// la table n'est pas definie, on cherche a completer
	trouve=findAttr(it,&nt,where[i].at2);
	switch (trouve) {
	case 0 : printf("Erreur dans la requete : attribut %s inconnu\n",where[i].at2);
	  exit(0);
	case 1 : where[i].table2=recopie(nt);
	  break;
	default : printf("Erreur dans la requete : attribut %s ambigu\n",where[i].at2);
	  exit(0);
	}
      }else {//la table est definie, on verifie qu'elle possede l'attribut indique
	if (getAlias(it,where[i].table2,where[i].at2)==NULL) {
	  printf("Erreur dans la requete : l'attribut %s n'est pas defini pour la table %s\n",where[i].at2,where[i].table2);
	  exit(0);
	}
      }
    }
  }
}
void controleAttr(info_tables* it,list_attr* select,int nbat) {
  int i;
  int trouve;
  char* nt;
  for (i=0;i<nbat;i++) {
    if (select[i].table==NULL) {// la table n'est pas definie, on cherche a completer
      trouve=findAttr(it,&nt,select[i].at);
      switch (trouve) {
      case 0 : printf("Erreur dans la requete : attribut %s inconnu\n",select[i].at);
	exit(0);
      case 1 : select[i].table=recopie(nt);
	break;
      default : printf("Erreur dans la requete : attribut %s ambigu\n",select[i].at);
	exit(0);
      }
    } else {//la table est definie, on verifie qu'elle possede l'attribut indique
      if (getAlias(it,select[i].table,select[i].at)==NULL) {
	printf("Erreur dans la requete : l'attribut %s n'est pas defini pour la table %s\n",select[i].at,select[i].table);
	exit(0);
      }
    }
  }
}
/* TODO : il faudra un jour prendre en compte le fait qu'il puisse y avoir des jointures multi criteres entre deux tables */
list_sql2 *creeArbre(list_op* where,int nbop,list_attr* select,int nbat,info_tables* it,char* from_att) {
  /* on commence par crer un ensemble de noeuds independants correspondant chacun a une operation elementaire */
  list_sql2* tete;
  list_sql2* new;
  list_tables* tu;
  int inseres;
  int numTable;
  int cpt;
  int rang;
  int type;
  int * util;
  int i;
  int numtable;

  controleTables(it,from_att);
  controleOps(it,where,nbop);
  controleAttr(it,select,nbat);

  /* ancienne classification 
  if (nbop==1) { //monojointure
    type=OP_JOIN_SIMPLE;
  } else { //multijointure
    type=OP_JOIN_MULTI;
  }
  */

  /* Assemblage de l'arbre */
  /* On commence par le premier noeud de la liste, et on ajoute les noeuds des qu'une de leurs tables
   fait partie des tables deja referencees dans l'arbre en cours de construction.
   Ceci assure la calculabilite des noeuds et la reutilisation des resultats produits en amont dans l'arbre */
  /* util idique si une operation de where a etee placee dans l'arbre */
  util=(int*) malloc(sizeof(int)*nbop);
  for (i=0;i<nbop;i++) {
    util[i]=0;
  }
  tu=(list_tables*)malloc(sizeof(list_tables));
  initTU(tu);

  tete=op2sql2(where[0],type,NULL,0,0,it);
  util[0]=1;
  ajoutTU(tu,where[0]);
  inseres=1;

  while (inseres<nbop) {
    cpt=rangInserable(where,util,tu,nbop,&numtable);
    /* TODO : un cas de produit cartesien... trrrres mal traite pour l'instant */
    if (cpt==-1) {
      printf("******ERREUR CPT *******\n");
      free(tu);
      free(util);
      return tete;
    }
    util[cpt]=1;
    ajoutTU(tu,where[cpt]);
    new=op2sql2(where[cpt],type,tete,numtable,inseres,it);
    new->suivant1=tete;
    tete=new;
    inseres++;
    //printf("inseres %d\n",inseres);
  }
  free(tu);

  ajoutNodes(tete);

  ajoutProj(it,tete,select,nbat);
  set_sql2_queries(tete);
  
  free(util);
  markAllUnused(it);
  return tete;
}

list_sql2* requete_creation(char* sql) {
  list_sql2* req=(list_sql2*)malloc(sizeof(list_sql2));
  req->type=OP_SCHEMA;
  req->requete_sql=recopie(sql);
  // tous les autres champs sont  NULL
  req->table1=NULL;
  req->table2=NULL;
  req->at1=NULL;
  req->at2=NULL;
  req->op_join=NULL;
  req->at1_res=NULL;
  req->at2_res=NULL;
  req->nb_at1_res=0;
  req->nb_at2_res=0;
  req->todelete=0;
  req->machine=NULL;
  req->suivant1=NULL;
  req->suivant2=NULL;

  return req;
}



/**
int main() {
  int i;
  int nbop;
  list_op* lop;
  int nbat;
  list_attr* la;
  list_sql2 *test,*test2;
  char* where_att=recopie("[1,=,athlete.num_ath,part.num_ath][1,=,comp.code_comp,part.code_comp]1");
  char* select_att=recopie("athlete.num_ath,athlete.nom,athlete.prenom,comp.code_comp,comp.date_comp");

  for (i=0;i<100;i++) {
    nbop=getOperations(where_att,&lop);
    //printOperations(lop,nbop);
    nbat=getAttributes(select_att,&la);
    //printAttributes(la,nbat);
    //printf("******************************************");
    test=creeArbre(lop,nbop,la,nbat);
    //printf("******************************************");
    test2=test;
    while(test2!=NULL) {
      trace_list_sql2(test2);
      test2=test2->suivant1;
    }
    freeOperations(&lop,nbop);
    freeAttributes(&la,nbat);
  }
}
**/
