
// Parallel database join header ... 
  
#ifndef _pdbml_h
#define _pdbml_h


#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <mpi.h>   
#include <mysql.h>
#include <pthread.h>
#include "pdbml_params.h"
#include "hist/hist.h"
#include "parser.h"
#include "subsite.h"

#define  Max_fields       40

#define  abs_(x) (x>0)?(x):-(x)

#define  OSFA_JOIN        0      // type d'algorithme de jointure a utiliser
#define  THREAD_OSFA_JOIN 1 
#define  PIPE_OSFA_JOIN   2 
#define  SFA_JOIN         3
#define  FA_JOIN          4
#define  HASH_JOIN        5
#define  ALL_JOINS        6

#define  SEND_BUFF_SIZE 8192       // Taille des blocs lors d'une operation de communication
#define  HIST_INIT_SIZE 8192       // Taille initiale a allouer lors de la creation d'un histogramme    

static pthread_mutex_t _mutex_=PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t _mutex_barrier_=PTHREAD_MUTEX_INITIALIZER;
static int      NODES_NUMBER=0;

typedef  char __String[256];
typedef  char __Query[1024];


typedef struct  {
          char   name[32];    // le nom de la machine
          char username[32];  // le nom de l'utilisateur Mysql
          char password[32];  // le mot de passe de l'utilisateur Mysql
          char bdname[32];    // le nom de la base Mysql  
          char   path[256];   // le nom de repertoire  sur chaque machine  pour stocker  les fichiers temporaires
          int    disk;        // le numero de  disk, sert a gerer le cas ou plusieurs machines se partagent le meme disque  0, 1, 2 ....
          int    port;        // le numero de port utilise'  pour  se coonecter au serveur mysql
          int    id;          // le numero de la machine           
     } node_struct ;

node_struct host[Max_nodes];


struct connect_ses {
  char *user;
  char *password;
  char *host;
  char *dbase;  
};

typedef struct connect_ses Connect_session;

struct thr_connect {
 Connect_session* connect;
 void   *params;
};

Connect_session session;




struct struct_global_osfa_distribution {
char     *name1,*name2,*field1,*field2;
int      nprocs,myid,threshold,*nodes_id;
MYSQL    *sock;
MPI_Comm Comm;
};


struct struct_init_histogram {
  char *name,*field;
  int   myid,thr_id,nprocs,*size,*length,op_idx;
  HIST  *tree,*hist,*ghist,*gcomm;
  // MATTHIEU HIST_ARRAY  **list,*clist,*hlist,**commlist,**hist_list;
  HIST_ARRAY  ***list,**clist,**hlist,***commlist,***hist_list;
  MYSQL *sock;
  MPI_Comm *comm; 
  Connect_session *session;
  int (*hash_func) (char*,int); 
  
};


struct struct_sfa_distribution {
     char *name1,*field,*name2,*name3,*name4;
     int   idx_num,nprocs,myid,*nodes_id,threshold,thr_id;
     HIST *gcomm,*hist;
     MYSQL *sock;
     MPI_Comm *comm; 
     Connect_session *session;    
     int (*hash_func) (char*,int);
};




int  host_id(char*);
int  new_host_id(char* ,node_struct*, int);

void init_hosts(node_struct* hosts, char* config_file , int* n );
void final_result_table_name(list_sql2*,MYSQL*,int, char*);



void show_result(MYSQL*,char*);
 

int connect_start(Connect_session*, MYSQL*);
void* startf(void *args);  
void local_table_name(char*,int, char*);

int sql_execute_query(char  *qbuf,MYSQL *sock);



int exist_table_name(char* name,MYSQL* sock);

int drop_table_name(char* name, MYSQL* sock);
int store_tmp_name(char* name,char* table, MYSQL* sock);
int drop_all_temporary_tables(char* table,MYSQL* sock);



int randomize(int max_value);
int hash_int(int i,int n);
int hash_string(char* key,int n);
int distribute_value(int n,int nprocs,int i,int myid);
 

void split_sort_int(int* src,int* index,int n);
void split_sort_int_right(int* src,int* index,int n);


// write_row ecrit un enregistrement d'une table dans un fichier  
int write_row(MYSQL_ROW  row,int nfields,FILE* fd);

int copy_struct(char* table1, char* table2,MYSQL* sock);



int copy_struct_idx(char* table1,char* table2,char* field,MYSQL* sock);


int copy_struct2_idx_offset(char* table1,char* table2,char* table3,char* field,int* nfields,MYSQL* sock);


int copy_struct3_idx_offset(char* table1,char* table2,char* table3,char* table4,char* field,int* nfields,MYSQL* sock);





void send_local_files(char* name1,char* name2,int myid,int nprocs,int* nodes_id,MPI_Comm com);
void send_double_idx_files(char* name1,char* name2,int myid,int nprocs,int* nodes_id,MPI_Comm com);



int load_local_files(char* sfile,char* table,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com) ;
int load_local_double_idx_files(char* sfile,char *table,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com); 



int exchange_local_files(char* table,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com) ;
void MPIT_Send(void* buf,int count,MPI_Datatype datatype,int dest,int tag, MPI_Comm comm);
void MPIT_Recv(void* buf,int count,MPI_Datatype datatype,int source,int tag, MPI_Comm comm,MPI_Status *status);
void MPIT_Barrier(MPI_Comm comm, int thr,int myid,int numprocs);



int exchange_double_idx_files(char* table,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com); 
int exchange_all_indexed_files(char* ftable,char* htable,char* dtable,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com); 
int thread_exchange_all_indexed_files(char* ftable,char* htable,char* dtable,int nprocs,int myid,int thr_id,int* nodes_id,MYSQL* sock,MPI_Comm com) ;
int exchange_all_indexed_files1(char* ftable,char* htable,char* dtable,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com);
int exchange_all_indexed_files2(char* ftable,char* htable,char* dtable,int nprocs,int myid,int* nodes_id,MYSQL* sock,MPI_Comm com); 

int create_local_histogram(char* table,char* field,HIST* hist_name,int myid,MYSQL* sock);

// MATTHIEU int local_histogram_array_buffer_distribution(HIST *hist,HIST_ARRAY **adlist,int nprocs,int myid, int hash_func(char*,int));
int local_histogram_array_buffer_distribution(HIST *hist,HIST_ARRAY ***adlist,int nprocs,int myid, int hash_func(char*,int));
//MATTHIEU int array_buffer_all_gather(HIST_ARRAY *src_list,HIST_ARRAY *dest_list,int nprocs,int myid,MPI_Comm com);
 void array_buffer_all_gather(HIST_ARRAY **src_list,HIST_ARRAY **dest_list,int nprocs,int myid,MPI_Comm com);
//MATTHIEU int array_buffer_exchange_gather(HIST_ARRAY *src_list,HIST_ARRAY *dest_list,int nprocs,int myid,MPI_Comm com);
int array_buffer_exchange_gather(HIST_ARRAY **src_list,HIST_ARRAY **dest_list,int nprocs,int myid,MPI_Comm com);

//MATTHIEU int  create_global_hash_histogram(HIST *tree,HIST *ghist,HIST_ARRAY list,int size,int length,int nprocs,int myid) ;
int  create_global_hash_histogram(HIST *tree,HIST *ghist,HIST_ARRAY *list,int size,int length,int nprocs,int myid) ;
//MATTHIEU int  array_buffer_communication_tree(HIST *tree,HIST *hist,HIST_ARRAY **dest_list,int nprocs,unsigned int idx_num,int myid);
int  array_buffer_communication_tree(HIST *tree,HIST *hist,HIST_ARRAY ***dest_list,int nprocs,unsigned int idx_num,int myid);
//MATTHIEU int  local_sjoin_histogram_array_buffer(HIST *tree,HIST *hist,HIST_ARRAY **hist_list,int nprocs);
int  local_sjoin_histogram_array_buffer(HIST *tree,HIST *hist,HIST_ARRAY ***hist_list,int nprocs);
// MATTHIEU int  RAND_local_sjoin_histogram_array_buffer(HIST *tree,HIST *hist,HIST_ARRAY **hist_list,int nprocs,comms *commEnv);
int  RAND_local_sjoin_histogram_array_buffer(HIST *tree,HIST *hist,HIST_ARRAY ***hist_list,int nprocs,comms *commEnv);
//MATTHIEU int array_buffer_load_global_communication(HIST_ARRAY list,HIST * tree,int size,int length);
int array_buffer_load_global_communication(HIST_ARRAY *list,HIST * tree,int size,int length);





void init_histograms(char* table1,char* field1,char* table2,char* field2,HIST* hist,HIST* gcomm1,HIST* gcomm2,MPI_Comm communicator,int num_procs,int threshold,int myid,MYSQL* sock,int      hash_func(char*,int));
void sfa_distribution_table(char* name1,char* field,unsigned int idx_num,int nprocs,HIST *gcomm,HIST *hist,int threshold,char* name2,char* name3,char* name4,int myid,int *nodes_id,MYSQL* sock,int hash_func(char*,int));

void global_sfa_distribution(char* name1,char* field1,char* name2,char* field2,int nprocs,int threshold,int myid,int *nodes_id,MYSQL* sock,MPI_Comm Comm,int hash_func(char*,int));

char* get_attribute_alias(char* att, char** attr_list,char** alias_list,int nb_attr);
char* create_join_query(list_sql2  *sql_query,int myid);
char* create_join_query_tablenames(list_sql2  *sql_query,char* table1,char* table2,char* table3,int myid);
char* create_append_join_query(list_sql2  *sql_query,int myid);
char* create_append_join_query_tablenames(list_sql2  *sql_query,char* table1,char* table2,char* table3,int myid);


 
void  show_sql_queries(query_list  *requetes);
double sfa_simple_join(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint* nodes_id,MYSQL* sock,MPI_Comm Comm,int hash_func(char*,int)) ;
double sfa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL    *sock,MPI_Comm Comm,int      hash_func(char*,int)) ;




void osfa_init_histograms(char* table1,char* field1,char* table2,char* field2,HIST *hist1,HIST *gcomm1,HIST *hist2,HIST *gcomm2,MPI_Comm communicator,int num_procs,int threshold,int myid,MYSQL* sock,int      hash_func(char*,int),int typeComm, comms *commEnv);

void global_osfa_distribution(char* name1,char* field1,char* name2,char* field2,int nprocs,int threshold,int myid,int *nodes_id,MYSQL* sock,MPI_Comm Comm,int hash_func(char*,int),int typeComm,comms *commEnv);

double osfa_simple_join(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint* nodes_id,MYSQL  *sock,MPI_Comm Comm,int   hash_func(char*,int)) ;
double osfa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL *sock,MPI_Comm Comm,int   hash_func(char*,int)) ;

void pipe_osfa_init_histograms(list_sql2 *sql_query,HIST *hist1,HIST *gcomm1,HIST *hist2,HIST *gcomm2,MPI_Comm communicator,int num_procs,int threshold,int myid,MYSQL* sock,int hash_func(char*,int));
void global_pipe_osfa_distribution(list_sql2* sql_query,int nprocs,int threshold,int myid,int *nodes_id,MYSQL* sock,MPI_Comm Comm,int hash_func(char*,int));

double pipe_osfa_simple_join(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint* nodes_id,MYSQL  *sock,MPI_Comm Comm,int   hash_func(char*,int)) ;
double pipe_osfa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL *sock,MPI_Comm Comm,int   hash_func(char*,int)) ;

//MATTHIEU void init_struct_histogram(char* name,char* field,int* size,int* length,int op_idx,int myid,int thr_id,int nprocs,HIST  *tree,HIST  *hist,HIST  *ghist,HIST  *gcomm,HIST_ARRAY **list,HIST_ARRAY *clist,HIST_ARRAY *hlist,HIST_ARRAY **commlist,HIST_ARRAY **hist_list,MYSQL* sock,MPI_Comm* comm,Connect_session *session,int        hash_func(char*,int),struct struct_init_histogram *st_hist);
void init_struct_histogram(char* name,char* field,int* size,int* length,int op_idx,int myid,int thr_id,int nprocs,HIST  *tree,HIST  *hist,HIST  *ghist,HIST  *gcomm,HIST_ARRAY ***list,HIST_ARRAY **clist,HIST_ARRAY **hlist,HIST_ARRAY ***commlist,HIST_ARRAY ***hist_list,MYSQL* sock,MPI_Comm* comm,Connect_session *session,int        hash_func(char*,int),struct struct_init_histogram *st_hist);


void* osfa_init_histogram_start(void*);
void* osfa_init_histogram_exchange_gather(void*);
void* osfa_hash_hist_start(void*);
void* new_osfa_init_histogram_start(void*);



void* osfa_communication_create(void*);
void* osfa_communication_create2(void*);
void* new_osfa_communication_create2(void*);
void* osfa_communication_finalize(void*);

void thread_osfa_init_histograms(char* table1,char* field1,char* table2,char* field2,HIST* hist1,HIST* gcomm1,HIST* hist2,HIST* gcomm2,MPI_Comm communicator,int num_procs,int threshold,int myid,MYSQL *thr_sock1,MYSQL *thr_sock2,int hash_func(char*,int));

void new_thread_osfa_init_histograms(char* table1,char* field1,char* table2,char* field2,HIST* hist1,HIST* gcomm1,HIST* hist2,HIST* gcomm2,MPI_Comm communicator,int num_procs,int threshold,int myid,MYSQL *thr_sock1,MYSQL  *thr_sock2,int hash_func(char*,int));

void init_struct_sfa_distribution(char  *name1,char  *field,int idx_num,int nprocs,HIST  *gcomm,HIST  *hist,int threshold,char* name2,char* name3,char* name4,int myid,int *nodes_id,MYSQL * sock,int   hash_func(char*,int),Connect_session *session,MPI_Comm *comm,int thr_id,struct struct_sfa_distribution *st_sfa);



void* osfa_distribution_start(void*);
void* osfa_distribution_start2(void*);
void thread_global_osfa_distribution(char* name1,char* field1,char* name2,char* field2,int nprocs,int threshold,int myid,int *nodes_id,MYSQL    *thread_sock1,MYSQL    *thread_sock2,MPI_Comm Comm,int      hash_func(char*,int));

double thread_osfa_simple_join(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL  *sock,MYSQL  *thread_sock1,MYSQL    *thread_sock2,MPI_Comm Comm,int hash_func(char*,int)); 
double threaded_osfa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint* nodes_id,MYSQL    *sock,MYSQL    *thread_sock1,MYSQL    *thread_sock2,MPI_Comm Comm,int      hash_func(char*,int)); 

double thread_osfa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint* nodes_id,MYSQL  *sock,MPI_Comm Comm,int  hash_func(char*,int)) ;
//MATTHIEU int  array_buffer_fa_communication_tree(HIST *tree,HIST *hist,HIST_ARRAY **dest_list,int nprocs,unsigned int threshold,int myid);
int  array_buffer_fa_communication_tree(HIST *tree,HIST *hist,HIST_ARRAY ***dest_list,int nprocs,unsigned int threshold,int myid);
void init_histogram_and_global_communication(char* table,char* field,HIST *hist,HIST  *gcomm,MPI_Comm communicator,int  num_procs,int threshold,int myid,MYSQL *sock,int  hash_func(char*,int));

// standart fa_join functions

void fa_distribution_build_table(char* name1,char* field,int nprocs,HIST *gcomm,HIST *hist,unsigned int threshold,char* name2,char* name3,int myid,int* nodes_id,MYSQL* sock,int hash_func(char*,int));
void fa_distribution_probe_table(char* name1,char* field, int nprocs,HIST *hist,unsigned intthreshold,char* name2,char* name3, int myid, int *nodes_id,MYSQL *sock,int hash_func(char*,int));
void global_fa_distribution(char* name1,char* field1,char* name2,char* field2,int nprocs,int threshold,int myid,int *nodes_id,MYSQL *sock,MPI_Comm Comm,int hash_func(char*,int));
double fa_simple_join(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL  *sock,MPI_Comm Comm,int hash_func(char*,int)); 
double fa_join_queries(list_sql2 *sql_query,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL *sock,MPI_Comm Comm,int hash_func(char*,int)) ;

// standart hash join functions  

void std_hash_distribution(char* name1,char* field,int nprocs,char* name2,int myid,int* nodes_id,MYSQL* sock, int hash_func(char*,int));
void global_std_hash_distribution(char* name1,char* field1,char* name2,char* field2,int nprocs,int myid,int* nodes_id,MYSQL *sock,MPI_Comm Comm,int hash_func(char*,int));
double hash_simple_join(list_sql2 *sql_query,uint nprocs,uint myid,uint *nodes_id,MYSQL    *sock,MPI_Comm  Comm,int hash_func(char*,int)) ;
double hash_join_queries(list_sql2 *sql_query,uint nprocs,uint myid,uint* nodes_id,MYSQL* sock,MPI_Comm Comm,int hash_func(char*,int)); 
double join_queries(query_list* requetes,uint threshold,uint nprocs,uint myid,uint *nodes_id,MYSQL *sock,MPI_Comm Comm,int  hash_func(char*,int),int join_type) ;

// 
int table_size(char* table, MYSQL* sock);
double  JP_average(double*src,int nprocs); 
double  JP_std_deviation(double* src, int nprocs);

int     Max_deviation(int* src); 
double  Max_deviation_d(double* src);
#endif

