#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include "infostables.h"

void parseheader(xmlDocPtr doc,xmlNodePtr curpar,info_table *tab) {
  xmlChar *base;
  xmlNodePtr cur;

  cur=curpar->xmlChildrenNode;
  while (cur!=NULL) {     
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"tablename"))) {
      tab->nom_table=(char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
    }
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"basename"))) {
      /*base=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
      printf("Nom de la base : %s (non exploite)\n",base);
      xmlFree(base);*/
    }
    cur=cur->next;
  }
  return;
}

void parseattributes(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlChar *type;
  xmlNodePtr curattribs;
  xmlNodePtr cur;


  table->nbattr=0;
  table->attribs=(info_attr*)malloc(sizeof(info_attr)*NBMAXATTR);

  curattribs=curpar->xmlChildrenNode;
  while (curattribs!=NULL) {     
    if ((!xmlStrcmp(curattribs->name,(const xmlChar *)"attribute"))) {
      cur=curattribs->xmlChildrenNode;
      while (cur!=NULL) {     
	if ((!xmlStrcmp(cur->name,(const xmlChar *)"attrname"))) {
	  table->attribs[table->nbattr].nom_attr=
	    (char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	}
	if ((!xmlStrcmp(cur->name,(const xmlChar *)"attrtype"))) {
	  /*type=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	  printf("Type de l'attribut : %s (non exploite) \n",type);
	  xmlFree(type);*/
	}
	cur=cur->next;
      }
      table->nbattr++;
    }
    curattribs=curattribs->next;
  }

  table->attribs=(info_attr*)realloc(table->attribs,sizeof(info_attr)*table->nbattr);
  return;
}

void parsemachines(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlChar *locname;
  xmlChar *tuples;
  xmlChar *size;
  int ituples;
  int isize;

  xmlNodePtr curmachines;
  xmlNodePtr cur;

  curmachines=curpar->xmlChildrenNode;

  table->nbnode=0;
  table->nodes=(char**)malloc(sizeof(char*)*NBMAXNODE);

  while (curmachines!=NULL) {     
    if ((!xmlStrcmp(curmachines->name,(const xmlChar *)"machine"))) {
      cur=curmachines->xmlChildrenNode;
      while (cur!=NULL) {     
	if ((!xmlStrcmp(cur->name,(const xmlChar *)"machinename"))) {
	  table->nodes[table->nbnode]=(char*)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	}
	if ((!xmlStrcmp(cur->name,(const xmlChar *)"machtuples"))) {
	  /*tuples=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	  ituples=atoi(tuples);
	  printf("nombre de tuples : %d (non exploite)\n",ituples);
	  xmlFree(tuples);*/
	}
	if ((!xmlStrcmp(cur->name,(const xmlChar *)"machsize"))) {
	  /*size=xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
	  isize=atoi(size);
	  printf("Taille du fragment : %d (non exploite)\n",isize);
	  xmlFree(size);*/
	}
	cur=cur->next;
      }
      table->nbnode++;
    }
    curmachines=curmachines->next;
  }

  table->nodes=(char**)realloc(table->nodes,sizeof(char*)*table->nbnode);

  return;
}

void parseTable(xmlDocPtr doc,xmlNodePtr curpar,info_table *table) {
  xmlNodePtr cur;

  cur=curpar->xmlChildrenNode;
  while (cur!=NULL) {     
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"header"))) {
      parseheader(doc,cur,table);
    }
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"attributes"))) {
      parseattributes(doc,cur,table);
    }
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"machines"))) {
      parsemachines(doc,cur,table);
    }
    cur=cur->next;
  }
  return;
}


void parseInfo(xmlDocPtr doc,xmlNodePtr cur,info_tables *it) {
  cur=cur->xmlChildrenNode;
  it->nbt=0;
  it->tabs=(info_table*)malloc(sizeof(info_table)*NBMAXTABLES);

  while (cur!=NULL) {
    if ((!xmlStrcmp(cur->name,(const xmlChar *)"table"))) {
      parseTable(doc,cur,&(it->tabs[it->nbt]));
      it->nbt++;
    }
    cur=cur->next;
  }

  it->tabs=(info_table*)realloc(it->tabs,sizeof(info_table)*it->nbt);

  return;
}

void addAliases(info_tables *it) {
  int numtab;
  int numatt;
  int numalias=0;
  for (numtab=0;numtab<it->nbt;numtab++) {
    for (numatt=0;numatt<it->tabs[numtab].nbattr;numatt++) {
      sprintf(it->tabs[numtab].attribs[numatt].alias_attr,"ATT%03d",numalias);
      numalias++;
    }
    it->tabs[numtab].lastAlias=it->tabs[numtab].attribs[numatt-1].alias_attr;
  }
}

void traceInfoTables(info_tables* it) {
  int i,j;

  for (i=0;i<it->nbt;i++) {  
    printf("Table #%i : %s\n",i, it->tabs[i].nom_table);
    for (j=0;j<it->tabs[i].nbattr;j++) {
      printf("Attribut #%i : %s (alias : %s)\n",j,it->tabs[i].attribs[j].nom_attr,it->tabs[i].attribs[j].alias_attr);
    }
    for (j=0;j<it->tabs[i].nbnode;j++) {
      printf("Noeud #%i : %s\n",j,it->tabs[i].nodes[j]);
    }
    printf("*****************\n");
  }
}

void main (int argc,char*argv[]){
  
  info_tables it;

  xmlDocPtr doc;
  xmlNodePtr cur;
  char *docname;

  docname=argv[1];
  doc=xmlParseFile(docname);

  if (doc==NULL) {
    fprintf(stderr,"Infotable not parsed\n");
    return;
  }

  cur=xmlDocGetRootElement(doc);

  if (cur==NULL) {
    fprintf(stderr,"Empty document\n");
    xmlFreeDoc(doc);
    return;
  }    

  parseInfo(doc,cur,&it);
  addAliases(&it);

  traceInfoTables(&it);
}
